/******************************************************
 * Linker script for sh-elf-ld v0.2 (w) Groepaz/Hitmen
 * Target CPU: Hitachi/SH4  Machine: Sega Dreamcast
 ******************************************************/

MEMORY {
    zeroram  : ORIGIN = 0x8c000000, LENGTH = 0x00010000   /*  64k */
    workram  : ORIGIN = 0x8c010000, LENGTH = 0x00de0000   /* ~14M */
    stacktop : ORIGIN = 0x8cdefffc, LENGTH = 4
    ddhram   : ORIGIN = 0x8cdf0000, LENGTH = 0x00010000   /*  64k */
    sysram   : ORIGIN = 0x8ce00000, LENGTH = 0x00200000   /*   2M */
    bootrom  : ORIGIN = 0xa0000000, LENGTH = 2M
    waveram  : ORIGIN = 0xa0800000, LENGTH = 2M
    videoram : ORIGIN = 0xa5000000, LENGTH = 8M
}

SECTIONS {

	/* ? */

  .interp        : { *(.interp)         }
  .hash          : { *(.hash)           }

  .dynsym        : { *(.dynsym)         }
  .dynstr        : { *(.dynstr)         }

	/* gnu version information */

  .gnu.version   : { *(.gnu.version)    }
  .gnu.version_d : { *(.gnu.version_d)  }
  .gnu.version_r : { *(.gnu.version_r)  }

  /* relocation information */

  .rel.text      : { *(.rel.text)
  		               *(.rel.text.*)
							*(.rel.gnu.linkonce.t*) }

  .rela.text     : { *(.rela.text)
  		               *(.rela.text.*)
							*(.rela.gnu.linkonce.t*) }

  .rel.data      : { *(.rel.data)
  		               *(.rel.data.*)
							*(.rel.gnu.linkonce.d*) }

  .rela.data     : { *(.rela.data)
  		               *(.rela.data.*)
							*(.rela.gnu.linkonce.d*) }

  .rel.rodata    : { *(.rel.rodata)
  		               *(.rel.rodata.*)
							*(.rel.gnu.linkonce.r*) }

  .rela.rodata   : { *(.rela.rodata)
  		               *(.rela.rodata.*)
							*(.rela.gnu.linkonce.r*) }

  .rel.got		  : { *(.rel.got) }
  .rela.got		  : { *(.rela.got) }

  .rel.ctors     : { *(.rel.ctors) }
  .rela.ctors    : { *(.rela.ctors) }

  .rel.ctors     : { *(.rel.ctors) }
  .rela.ctors    : { *(.rela.ctors) }

  .rel.init      : { *(.rel.init) }
  .rela.init     : { *(.rela.init) }

  .rel.fini      : { *(.rel.fini) }
  .rela.fini     : { *(.rela.fini) }

  .rel.bss       : { *(.rel.bss) }
  .rela.bss      : { *(.rela.bss) }

  .rel.plt       : { *(.rel.plt) }
  .rela.plt      : { *(.rela.plt) }

  /* ? */

  .init          : { *(.init) }
  .plt           : { *(.plt) }

  /* program layout information */

  .text : {
    	*(.text)                           /* code */
		*(.text.*)
		*(.stub)                           /* debug-stub code */
		*(.gnu.warning)
		*(gnu.linkonce.t*)

		_etext = ABSOLUTE(.) ;             /* => provide adr. of end of text */
    } > workram

  .fini : { *(.fini) }

  .data : {
		/* .rodata */
    	*(.rodata)                         /* initialized data */
		*(.rodata.*)
		*(.gnu.linkonce.r*)
		/* .rodata1 */
		*(.rodata1)
		/* .data */
		*(.data)

		*(.data.*)
		*(.gnu.linkonce.d*)
		/* .data1 */
		*(.data1)

		/* .ctors */
		. = ALIGN(4);
    	__ctors = ABSOLUTE(.) ;
   	*(.ctors)                          /* c++ constructors */
   	__ctors_end = ABSOLUTE(.) ;

		/* .dtors */
   	. = ALIGN(4);
    	__dtors = ABSOLUTE(.) ;
   	*(.dtors)                          /* c++ destructors */
   	__dtors_end = ABSOLUTE(.) ;

		_edata = ABSOLUTE(.) ;
    } > workram

  .got : { *(.got.plt)                   /* ? */
			  *(.got)     }

  .dynamic : { *(.dynamic) }             /* ? */

  .sdata : {
		/* .sdata */
		*(.sdata)
		*(.sdata.*)
		/* .sbss */
		*(.sbss)
		*(.scommon)
  }

  .bss (NOLOAD) : {                          /* non-initialized data */
    	*(dynbss)
		*(.bss)
		*(.bss.*)
		*(COMMON) 
    	_end = ABSOLUTE(.) ;
    } > workram

                                    /* <=- the heap will be here */

  .stack (NOLOAD) : {
    	*(.stack)                              /* cpu-stack */
    	_stack = ABSOLUTE(.) ;
    } > stacktop

  .stab 				: { *(.stab)          }             /* ? */
  .stabstr 			: { *(.stabstr)       }             /* ? */
  .stab.excl 		: { *(.stab.excl) 	 }             /* ? */
  .stab.exclstr 	: { *(.stab.exclstr)  }             /* ? */
  .stab.index 		: { *(.stab.index)    }             /* ? */
  .stab.indexstr  : { *(.stab.indexstr) }             /* ? */

  /* debug information */

  .comment 			: { *(.comment)       }             /* ? */
  .debug 			: { *(.debug)         }             /* ? */
  .line 				: { *(.line)          }             /* ? */

  .debug_srcinfo    : {  *(.debug_srcinfo)   }
  .debug_sfnames    : {  *(.debug_sfnames)   }
  .debug_aranges    : {  *(.debug_aranges)   }
  .debug_pubnames   : {  *(.debug_pubnames)  }
  .debug_info       : {  *(.debug_info)      }
  .debug_abbrev     : {  *(.debug_abbrev)    }
  .debug_line       : {  *(.debug_line)      }
  .debug_frame      : {  *(.debug_frame)     }
  .debug_str        : {  *(.debug_str)       }
  .debug_loc        : {  *(.debug_loc)       }
  .debug_macinfo    : {  *(.debug_macinfo)   }
  .debug_weaknames  : {  *(.debug_weaknames) }
  .debug_funcnames  : {  *(.debug_funcnames) }
  .debug_typenames  : {  *(.debug_typenames) }
  .debug_varnames   : {  *(.debug_varnames)  }

}
