

@--------------------------------------------------------------------
@-                            Constants                             -
@--------------------------------------------------------------------

ON                      =   1
OFF                     =   0
ENABLE                  =   1
DISABLE                 =   0
TRUE                    =   1
FALSE                   =   0

LCD_WIDTH               =   240         @ Screen Width
LCD_HEIGHT              =   160         @ Screen Height


@--------------------------------------------------------------------
@-                Register & RAM Reset System Call                  -
@--------------------------------------------------------------------

RESET_ALL_FLAG          =   0xFF        @ Reset All

RESET_ALL_REG_FLAG      =   0xe0        @ Reset all registers

RESET_REG_FLAG          =   0x80        @ Reset Register (Other than stated below)
RESET_REG_SOUND_FLAG    =   0x40        @ Reset Sound Register
RESET_REG_SIO_FLAG      =   0x20        @ Reset SIO Register

RESET_ALL_RAM_FLAG      =   0x1f        @ Clear the entire RAM

RESET_EX_WRAM_FLAG      =   0x01        @ Clear CPU External RAM
RESET_CPU_WRAM_FLAG     =   0x02        @ Clear CPU Internal RAM
RESET_PLTT_FLAG         =   0x04        @ Clear Palette
RESET_VRAM_FLAG         =   0x08        @ Clear VRAM
RESET_OAM_FLAG          =   0x10        @ Clear OAM


@--------------------------------------------------------------------
@-                    BG Screen Data                                -
@--------------------------------------------------------------------

BG_SC_CHAR_NO_MASK      =   0x03ff      @ Character No
BG_SC_FLIP_MASK         =   0x0c00      @ Flip
BG_SC_PLTT_MASK         =   0xf000      @ Color Palette No

BG_SC_CHAR_NO_SHIFT     =   0
BG_SC_FLIP_SHIFT        =   10
BG_SC_PLTT_SHIFT        =   12

BG_SC_H_FLIP            =   0x0400      @ H Flip
BG_SC_V_FLIP            =   0x0800      @ V Flip

@--------------------------------------------------------------------
@-                        Affine Transformation                     -
@--------------------------------------------------------------------

SIN_SHIFT               =   14
AFFINE_DIV_SHIFT        =   8
AFFINE_REG_SHIFT        =   8
AFFINE_CAL_SHIFT        =   SIN_SHIFT + AFFINE_DIV_SHIFT - AFFINE_REG_SHIFT

@--------------------------------------------------------------------
@-                        OAM Data                             -
@--------------------------------------------------------------------

OAM_V_POS_MASK          =   0x000000ff  @ OBJ X Coordinate
OAM_H_POS_MASK          =   0x01ff0000  @ OBJ Y Coordinate
OAM_SHAPE_MASK          =   0x0000c000  @ OBJ Shape
OAM_SIZE_MASK           =   0xc0000000  @ OBJ Size
OAM_OBJ_MODE_MASK       =   0x00000c00  @ OBJ Mode
OAM_AFFINE_MODE_MASK    =   0x00000300  @ Affine Mode
OAM_AFFINE_NO_MASK      =   0x3e000000  @ Affine Parameter No
OAM_FLIP_MASK           =   0x30000000  @ Flip
OAM_CHAR_NO_MASK        =   0x000003ff  @ Character No
OAM_PRIORITY_MASK       =   0x00000c00  @ BG Relative Priority
OAM_PLTT_MASK           =   0x0000f000  @ Color Palette No
OAM_AFFINE_PARAM_MASK   =   0xffff0000  @ Affine Parameter

OAM_V_POS_SHIFT         =   0
OAM_H_POS_SHIFT         =   16
OAM_SHAPE_SHIFT         =   14
OAM_SIZE_SHIFT          =   30
OAM_AFFINE_MODE_SHIFT   =   8
OAM_OBJ_MODE_SHIFT      =   10
OAM_AFFINE_NO_SHIFT     =   25
OAM_FLIP_SHIFT          =   28
OAM_CHAR_NO_SHIFT       =   0
OAM_PRIORITY_SHIFT      =   10
OAM_PLTT_SHIFT          =   12
OAM_AFFINE_PARAM_SHIFT  =   16

OAM_OBJ_ON              =   0x00000000  @ OBJ ON
OAM_OBJ_OFF             =   0x00000200  @ OBJ OFF
OAM_OBJ_NORMAL          =   0x00000000  @ OBJ Normal Mode
OAM_OBJ_BLEND           =   0x00000400  @ OBJ Semi-transparent Mode
OAM_OBJ_WINDOW          =   0x00000800  @ OBJ Window Mode
OAM_AFFINE_NONE         =   0x00000000  @ Affine Invalid Mode
OAM_AFFINE_NORMAL       =   0x00000100  @ Affine Normal Mode
OAM_AFFINE_TWICE        =   0x00000300  @ Affine Double-size Mode
OAM_MOS_ON              =   0x00001000  @ Mosaic ON
OAM_MOS_OFF             =   0x00000000  @ Mosaic OFF
OAM_COLOR_16            =   0x00000000  @ Select 16
OAM_COLOR_256           =   0x00002000  @ Select 256 Color
OAM_H_FLIP              =   0x10000000  @ H Flip
OAM_V_FLIP              =   0x20000000  @ V Flip
OAM_SQUARE              =   0x00000000  @ Square OBJ
OAM_H_RECTANGLE         =   0x00004000  @ Horizontally-oriented rectangle OBJ
OAM_V_RECTANGLE         =   0x00008000  @ Vertically-oriented rectangle OBJ
OAM_SIZE_NO_0           =   0x00000000  @ OBJ size No 0
OAM_SIZE_NO_1           =   0x40000000  @ OBJ size No 1
OAM_SIZE_NO_2           =   0x80000000  @ OBJ size No 2
OAM_SIZE_NO_3           =   0xc0000000  @ OBJ size No 3
OAM_SIZE_8x8            =   0x00000000  @ OBJ 8 x 8 dot
OAM_SIZE_16x16          =   0x40000000  @ OBJ 16 x 16 dot
OAM_SIZE_32x32          =   0x80000000  @ OBJ 32 x 32 dot
OAM_SIZE_64x64          =   0xc0000000  @ OBJ 64 x 64 dot
OAM_SIZE_16x8           =   0x00004000  @ OBJ 16 x 8 dot
OAM_SIZE_32x8           =   0x40004000  @ OBJ 32 x 8 dot
OAM_SIZE_32x16          =   0x80004000  @ OBJ 32 x 16 dot
OAM_SIZE_64x32          =   0xc0004000  @ OBJ 64 x 32 dot
OAM_SIZE_8x16           =   0x00008000  @ OBJ 8 x 16 dot
OAM_SIZE_8x32           =   0x40008000  @ OBJ 8 x 32 dot
OAM_SIZE_16x32          =   0x80008000  @ OBJ 16 x 32 dot
OAM_SIZE_32x64          =   0xc0008000  @ OBJ 32 x 64 dot


@--------------------------------------------------------------------
@-                      Color Palette Data                          -
@--------------------------------------------------------------------

PLTT_RED_MASK           =   0x001f      @ Red
PLTT_GREEN_MASK         =   0x03e0      @ Green
PLTT_BLUE_MASK          =   0x7c00      @ Blue

PLTT_RED_SHIFT          =   0
PLTT_GREEN_SHIFT        =   5
PLTT_BLUE_SHIFT         =   10


@********************************************************************
@*             AGB Register Control Bit                             *
@********************************************************************

@--------------------------------------------------------------------
@-                  Program Status Register                         -
@--------------------------------------------------------------------

PSR_CPU_MODE_MASK       =   0x1f        @ CPU Mode

PSR_USER_MODE           =   0x10        @ User
PSR_FIQ_MODE            =   0x11        @ FIQ
PSR_IRQ_MODE            =   0x12        @ IRQ
PSR_SVC_MODE            =   0x13        @ Supervisor
PSR_ABORT_MODE          =   0x17        @ Abort (Prefetch/Data)
PSR_UNDEF_MODE          =   0x1b        @ Undefined Command
PSR_SYS_MODE            =   0x1f        @ System

PSR_THUMB_STATE         =   0x20        @ THUMB State
PSR_FIQ_DISABLE         =   0x40        @ FIQ Disable
PSR_IRQ_DISABLE         =   0x80        @ IRQ Disable
PSR_IRQ_FIQ_DISABLE     =   0xc0        @ IRQ & FIQ Disable

PSR_V_FLAG              =   0x10000000  @ Overflow
PSR_C_FLAG              =   0x20000000  @ Carry/Borrow/Expand
PSR_Z_FLAG              =   0x40000000  @ Zero
PSR_N_FLAG              =   0x80000000  @ Negative/Less than


@--------------------------------------------------------------------
@-                  Interrupt Request/Enable Flag                   -
@--------------------------------------------------------------------

V_BLANK_INTR_FLAG       =   0x0001      @ V Blank Interrupt
H_BLANK_INTR_FLAG       =   0x0002      @ H Blank Interrupt
V_COUNT_INTR_FLAG       =   0x0004      @ V Counter Corresponding Interrupt
TIMER0_INTR_FLAG        =   0x0008      @ Timer 0 Interrupt
TIMER1_INTR_FLAG        =   0x0010      @ Timer 1 Interrupt
TIMER2_INTR_FLAG        =   0x0020      @ Timer 2 Interrupt
TIMER3_INTR_FLAG        =   0x0040      @ Timer 3 Interrupt
SIO_INTR_FLAG           =   0x0080      @ Serial Communication Interrupt
DMA0_INTR_FLAG          =   0x0100      @ DMA 0 Interrupt
DMA1_INTR_FLAG          =   0x0200      @ DMA 1 Interrupt
DMA2_INTR_FLAG          =   0x0400      @ DMA 2 Interrupt
DMA3_INTR_FLAG          =   0x0800      @ DMA 3 Interrupt
KEY_INTR_FLAG           =   0x1000      @ Key Interrupt
CASSETTE_INTR_FLAG      =   0x2000      @ Game Pak Interrupt


@--------------------------------------------------------------------
@-                      Display Control                             -
@--------------------------------------------------------------------

DISP_BG_MODE_MASK       =   0x0007      @ BG Mode
DISP_ON_MASK            =   0x1f00      @ OBJ, BG ON
DISP_WIN_MASK           =   0x6000      @ Window ON

DISP_BG_MODE_SHIFT      =   0
DISP_ON_SHIFT           =   8
DISP_WIN_SHIFT          =   13

DISP_MODE_0             =   0x0000      @ BG Mode 0
DISP_MODE_1             =   0x0001      @ BG Mode 1
DISP_MODE_2             =   0x0002      @ BG Mode 2
DISP_MODE_3             =   0x0003      @ BG Mode 3
DISP_MODE_4             =   0x0004      @ BG Mode 4
DISP_MODE_5             =   0x0005      @ BG Mode 5
DISP_BMP_FRAME_NO       =   0x0010      @ Bitmap Mode Display Frame
DISP_OBJ_HOFF           =   0x0020      @ OBJ Processing in H Blank OFF
DISP_OBJ_CHAR_2D_MAP    =   0x0000      @ OBJ Character Data 2D Mapping
DISP_OBJ_CHAR_1D_MAP    =   0x0040      @ OBJ Character Data 1D Mapping
DISP_LCDC_OFF           =   0x0080      @ LCDC OFF
DISP_BG0_ON             =   0x0100      @ BG0 ON
DISP_BG1_ON             =   0x0200      @ BG1 ON
DISP_BG2_ON             =   0x0400      @ BG2 ON
DISP_BG3_ON             =   0x0800      @ BG3 ON
DISP_BG_ALL_ON          =   0x0f00      @ All BG ON
DISP_OBJ_ON             =   0x1000      @ OBJ ON
DISP_OBJ_BG_ALL_ON      =   0x1f00      @ All OBJ, BG ON
DISP_WIN0_ON            =   0x2000      @ Window 0 ON
DISP_WIN1_ON            =   0x4000      @ Window 1 ON
DISP_WIN01_ON           =   0x6000      @ Window 0, 1 ON
DISP_OBJWIN_ON          =   0x8000      @ OBJ Window ON
DISP_WIN_ALL_ON         =   0xe000      @ All Window ON
DISP_ALL_ON             =   0x7f00      @ All ON

@--------------------------------------------------------------------
@-                      Status Control                              -
@--------------------------------------------------------------------

STAT_VCOUNT_CMP_MASK    =   0xff00      @ V Counter Comparison Value

STAT_VCOUNT_CMP_SHIFT   =   8

STAT_V_BLANK            =   0x0001      @ During V Blank
STAT_H_BLANK            =   0x0002      @ During H Blank
STAT_V_COUNT            =   0x0004      @ Matching V Counter
STAT_V_BLANK_IF_ENABLE  =   0x0008      @ V Blank Interrupt Request Enable
STAT_H_BLANK_IF_ENABLE  =   0x0010      @ H Blank Interrupt Request Enable
STAT_V_COUNT_IF_ENABLE  =   0x0020      @ V Counter Matching Interrupt Request Enable

@--------------------------------------------------------------------
@-                      BG Control                                  -
@--------------------------------------------------------------------

BG_PRIORITY_MASK        =   0x0003      @ BG Priority
BG_CHAR_BASE_MASK       =   0x000c      @ Character Base Address
BG_SCREEN_BASE_MASK     =   0x1f00      @ Screen Base Address
BG_SCREEN_SIZE_MASK     =   0xc000      @ Screen Size (256/512)

BG_PRIORITY_SHIFT       =   0
BG_CHAR_BASE_SHIFT      =   2
BG_SCREEN_BASE_SHIFT    =   8
BG_SCREEN_SIZE_SHIFT    =   14

BG_PRIORITY_0           =   0x0000      @ BG 0 Priority
BG_PRIORITY_1           =   0x0001      @ BG 1st Priority
BG_PRIORITY_2           =   0x0002      @ BG 2nd Priority
BG_PRIORITY_3           =   0x0003      @ BG 3rd Priority
BG_MOS_ON               =   0x0040      @ Mosaic ON
BG_MOS_OFF              =   0x0000      @ Mosaic OFF
BG_COLOR_16             =   0x0000      @ Select 16 colors
BG_COLOR_256            =   0x0080      @ Select 256 colors
BG_LOOP_ON              =   0x2000      @ Loop ON
BG_LOOP_OFF             =   0x0000      @ Loop OFF
BG_SCREEN_SIZE_0        =   0x0000      @ Screen Size (256x256)
BG_SCREEN_SIZE_1        =   0x4000      @ Screen Size (512x256)
BG_SCREEN_SIZE_2        =   0x8000      @ Screen Size (256x512)
BG_SCREEN_SIZE_3        =   0xc000      @ Screen Size (512x512)

@--------------------------------------------------------------------
@-                      Window Control                              -
@--------------------------------------------------------------------

WIN_END_POS_MASK        =   0x00ff      @ Window End Position
WIN_START_POS_MASK      =   0xff00      @ Window Start Position

WIN_END_POS_SHIFT       =   0
WIN_START_POS_SHIFT     =   8

WIN_BG0_ON              =   0x0001      @ BG0 ON
WIN_BG1_ON              =   0x0002      @ BG1 ON
WIN_BG2_ON              =   0x0004      @ BG2 ON
WIN_BG3_ON              =   0x0008      @ BG3 ON
WIN_OBJ_ON              =   0x0010      @ OBJ ON
WIN_BLEND_ON            =   0x0020      @ Blend ON
WIN_ALL_ON              =   0x003f      @ All ON

@--------------------------------------------------------------------
@-                      Mosaic Control                              -
@--------------------------------------------------------------------

MOS_H_SIZE_MASK         =   0x000f      @ Mosaic Height
MOS_V_SIZE_MASK         =   0x00f0      @ Mosaic Width
MOS_BG_H_SIZE_MASK      =   0x000f      @ BG Mosaic Height
MOS_BG_V_SIZE_MASK      =   0x00f0      @ BG Mosaic Width
MOS_OBJ_H_SIZE_MASK     =   0x000f      @ OBJ Mosaic Height
MOS_OBJ_V_SIZE_MASK     =   0x00f0      @ OBJ Mosaic Width

MOS_H_SIZE_SHIFT        =   0
MOS_V_SIZE_SHIFT        =   4
MOS_BG_H_SIZE_SHIFT     =   0
MOS_BG_V_SIZE_SHIFT     =   4
MOS_OBJ_H_SIZE_SHIFT    =   8
MOS_OBJ_V_SIZE_SHIFT    =   12

@--------------------------------------------------------------------
@-                      Blend Control                               -
@--------------------------------------------------------------------

BLD_PIXEL_MASK          =   0x003f      @ Select Pixel
BLD_1ST_PIXEL_MASK      =   0x003f      @ Select 1st Pixel
BLD_MODE_MASK           =   0x00c0      @ Blend Mode
BLD_2ND_PIXEL_MASK      =   0x3f00      @ Select 2nd Pixel

BLD_1ST_PIXEL_SHIFT     =   0
BLD_MODE_SHIFT          =   6
BLD_2ND_PIXEL_SHIFT     =   8

BLD_BG0                 =   0x0001      @ Select BG 0 Pixel
BLD_BG1                 =   0x0002      @ Select BG 1 Pixel
BLD_BG2                 =   0x0004      @ Select BG 2 Pixel
BLD_BG3                 =   0x0008      @ Select BG 3 Pixel
BLD_OBJ                 =   0x0010      @ Select OBJ Pixel
BLD_BD                  =   0x0020      @ Select Background Color Pixel
BLD_ALL                 =   0x003f      @ Select All Pixel
BLD_BG0_1ST             =   0x0001      @ Select BG 0 1st Pixel
BLD_BG1_1ST             =   0x0002      @ Select BG 1 1st Pixel
BLD_BG2_1ST             =   0x0004      @ Select BG 2 1st Pixel
BLD_BG3_1ST             =   0x0008      @ Select BG 3 1st Pixel
BLD_OBJ_1ST             =   0x0010      @ Select OBJ 1st Pixel
BLD_BD_1ST              =   0x0020      @ Select Background Color 1st Pixel
BLD_1ST_ALL             =   0x003f      @ Select All 1st Pixel
BLD_NORMAL_MODE         =   0x0000      @ Normal Mode
BLD_A_BLEND_MODE        =   0x0040      @ Semi-transparent Mode
BLD_UP_MODE             =   0x0080      @ Brightness UP Mode
BLD_DOWN_MODE           =   0x00c0      @ Brightness DOWN Mode
BLD_BG0_2ND             =   0x0100      @ Select BG 0 2nd Pixel
BLD_BG1_2ND             =   0x0200      @ Select BG 1 2nd Pixel
BLD_BG2_2ND             =   0x0400      @ Select BG 2 2nd Pixel
BLD_BG3_2ND             =   0x0800      @ Select BG 3 2nd Pixel
BLD_OBJ_2ND             =   0x1000      @ Select OBJ 2nd Pixel
BLD_BD_2ND              =   0x2000      @ Select Background Color 2nd Pixel
BLD_2ND_ALL             =   0x3f00      @ Select All 2nd Pixel


BLD_A_MASK              =   0x001f      @ Blend Variable A
BLD_B_MASK              =   0x1f00      @ Blend Variable B
BLD_Y_MASK              =   0x001f      @ Blend Variable Y

BLD_A_SHIFT             =   16
BLD_B_SHIFT             =   24
BLD_Y_SHIFT             =   0


@--------------------------------------------------------------------
@-                     Sound Control                                -
@--------------------------------------------------------------------

SOUND_SO1_LEVEL_MASK    =   0x03        @ S01 Output Level
SOUND_SO2_LEVEL_MASK    =   0x30        @ S02 Output Level
SOUND_DMG_SO1_ON_MASK   =   0x0f        @ DMG Compatible Sound ->S01 ON
SOUND_DMG_SO2_ON_MASK   =   0xf0        @ DMG Compatible Sound ->S02 ON

SOUND_SWEEP_SHIFT_MASK  =   0x03        @ Sweep Shift Number
SOUND_SWEEP_TIME_MASK   =   0x30        @ Sweep Time
SOUND_COUNTS_MASK       =   0x3f        @ Sound Length (Count Number)
SOUND_DUTY_MASK         =   0xc0        @ Waveform Duty
SOUND_ENV_STEPS_MASK    =   0x03        @ Envelope Step Number
SOUND_ENV_INIT_MASK     =   0xf0        @ Envelope Initial-value
SOUND_FREQUENCY_MASK    =   0x03ff      @ Frequency
SOUND_FREQUENCY_L_MASK  =   0xff
SOUND_FREQUENCY_H_MASK  =   0x03

SOUND_3_COUNTS_MASK     =   0xff        @ Sound 3  Sound Length (Count Number)
SOUND_3_LEVEL_MASK      =   0xe0        @            Output Level

SOUND_4_PRESCALER_MASK  =   0x03        @ Sound 4 Prescaler Select
SOUND_4_POLYSHIFT_MASK  =   0xf0        @       Polynomical Counter Shift Number


SOUND_SO1_LEVEL_SHIFT   =   0
SOUND_SO2_LEVEL_SHIFT   =   4
SOUND_DMG_SO1_ON_SHIFT  =   0
SOUND_DMG_SO2_ON_SHIFT  =   4

SOUND_SWEEP_SHIFT_SHIFT =   0
SOUND_SWEEP_TIME_SHIFT  =   4
SOUND_COUNTS_SHIFT      =   0
SOUND_DUTY_SHIFT        =   6
SOUND_ENV_STEPS_SHIFT   =   0
SOUND_ENV_INIT_SHIFT    =   4
SOUND_FREQUENCY_SHIFT   =   0
SOUND_FREQUENCY_L_SHIFT =   0
SOUND_FREQUENCY_H_SHIFT =   8

SOUND_3_COUNTS_SHIFT    =   0
SOUND_3_LEVEL_SHIFT     =   5

SOUND_4_PRESCALER_SHIFT =   0
SOUND_4_POLYSHIFT_SHIFT =   4


SOUND_1_ON              =   0x01        @ Sound 1 ON
SOUND_2_ON              =   0x02        @ Sound 2 ON
SOUND_3_ON              =   0x04        @ Sound 3 ON
SOUND_4_ON              =   0x08        @ Sound 4 ON
SOUND_DMG_ON            =   0x80        @ DMG Compatible Sound ON

SOUND_1_SO1_ON          =   0x01        @ Sound 1 -> S01 ON
SOUND_2_SO1_ON          =   0x02        @ Sound 2 -> S01 ON
SOUND_3_SO1_ON          =   0x04        @ Sound 3 -> S01 ON
SOUND_4_SO1_ON          =   0x08        @ Sound 4 -> S01 ON
SOUND_1_SO2_ON          =   0x10        @ Sound 1 -> S02 ON
SOUND_2_SO2_ON          =   0x20        @ Sound 2 -> S02 ON
SOUND_3_SO2_ON          =   0x40        @ Sound 3 -> S02 ON
SOUND_4_SO2_ON          =   0x80        @ Sound 4 -> S02 ON
SOUND_DMG_ALL_SO1_ON    =   0x0f        @ DMG Compatible Sound -> S01 ON
SOUND_DMG_ALL_SO2_ON    =   0xf0        @ DMG Compatible Sound -> S02 ON
SOUND_DMG_ALL_SO_ON     =   0xff        @ DMG Compatible Sound -> S01/2 ON
SOUND_VIN_SO1_ON        =   0x80        @ Vin -> S01 ON
SOUND_VIN_SO2_ON        =   0x08        @ Vin -> S02 ON
SOUND_VIN_SO_ON         =   0x08        @ Vin -> S01/2 ON

SOUND_A_SO1_ON          =   0x01        @ Direct Sound A -> S01 ON
SOUND_A_SO2_ON          =   0x02        @ Direct Sound A -> S02 ON
SOUND_B_SO1_ON          =   0x10        @ Direct Sound B -> S01 ON
SOUND_B_SO2_ON          =   0x20        @ Direct Sound B -> S02 ON
SOUND_A_ALL_SO_ON       =   0x03        @ Direct Sound A -> S01/2 ON
SOUND_B_ALL_SO_ON       =   0x30        @ Direct Sound B -> S01/2 ON
SOUND_DIRECT_ALL_SO1_ON =   0x11        @ Direct Sound A/B -> S01 ON
SOUND_DIRECT_ALL_SO2_ON =   0x22        @ Direct Sound A/B -> S02 ON
SOUND_DIRECT_ALL_SO_ON  =   0x33        @ Direct Sound A/B -> S01/2 ON
SOUND_A_TIMER_0         =   0x00        @ Timer 0 -> Direct Sound A
SOUND_A_TIMER_1         =   0x04        @ Timer 1 -> Direct Sound A
SOUND_B_TIMER_0         =   0x00        @ Timer 0 -> Direct Sound B
SOUND_B_TIMER_1         =   0x40        @ Timer 1 -> Direct Sound B
SOUND_A_FIFO_RESET      =   0x08        @ Reset Direct Sound A FIFO
SOUND_B_FIFO_RESET      =   0x80        @ Reset Direct Sound B FIFO

SOUND_DMG_MIX_1_4       =   0x00        @ DMG Compatible Sound Mix Ratio 1/4
SOUND_DMG_MIX_2_4       =   0x01        @                           2/4
SOUND_DMG_MIX_FULL      =   0x02        @                            Full Range
SOUND_A_MIX_1_2         =   0x00        @ Direct Sound A Mix Ratio 1/2
SOUND_A_MIX_FULL        =   0x04        @                            Full Range
SOUND_B_MIX_1_2         =   0x00        @ Direct Sound B Mix Ratio 1/2
SOUND_B_MIX_FULL        =   0x08        @                            Full Range
SOUND_ALL_MIX_FULL      =   0x0e        @ All Sound Mix Ratio Full Range


SOUND_INIT_ON           =   0x80        @ Initialize ON
SOUND_INIT_OFF          =   0x00        @ Initialize ON
SOUND_COUNTER_ON        =   0x40        @ Counter ON
SOUND_COUNTER_OFF       =   0x00        @ Counter OFF

SOUND_SWEEP_UP          =   0x00        @ Sweep Addition
SOUND_SWEEP_DOWN        =   0x08        @          Subtraction
SOUND_DUTY_1_8          =   0x00        @ Waveform Duty 1/8
SOUND_DUTY_2_8          =   0x40        @                  2/8
SOUND_DUTY_4_8          =   0x80        @                  4/8
SOUND_DUTY_6_8          =   0xc0        @                  6/8
SOUND_ENV_UP            =   0x08        @ Envelope UP
SOUND_ENV_DOWN          =   0x00        @              DOWN

SOUND_3_LOCAL_ON        =   0x80        @ Sound 3 ON
SOUND_3_WAVE_BANK_0     =   0x00        @ Sound 3 Waveform RAM Bank 0
SOUND_3_WAVE_BANK_1     =   0x40        @                       Bank 1
SOUND_3_WAVE_CONNECT    =   0x20        @ Sound 3 Waveform RAM Connect
SOUND_3_WAVE_DISCONNECT =   0x00        @                       Disconnect
SOUND_3_LEVEL_0_4       =   0x00        @ Sound 3 Output None
SOUND_3_LEVEL_FULL      =   0x20        @                 4/4
SOUND_3_LEVEL_2_4       =   0x40        @                 2/4
SOUND_3_LEVEL_1_4       =   0x60        @                 1/4
SOUND_3_LEVEL_3_4       =   0x80        @                 3/4

SOUND_4_POLYSTEP_15     =   0x00        @ Sound 4 Polynomical Counter 15 step
SOUND_4_POLYSTEP_7      =   0x08        @                              7 step


@--------------------------------------------------------------------
@-                      SIO Control                                 -
@--------------------------------------------------------------------

SIO_MODE_MASK           =   0x3000      @ Communication mode
SIO_BAUD_RATE_MASK      =   0x0003      @ Baud rate
SIO_ID_NO_MASK          =   0x0030      @ Communication ID

SIO_MODE_SHIFT          =   12
SIO_BAUD_RATE_SHIFT     =   0
SIO_ID_NO_SHIFT         =   4

SIO_8BIT_MODE           =   0x0000      @ Normal 8-bit communication mode
SIO_32BIT_MODE          =   0x1000      @ Normal 32-bit communication mode
SIO_MULTI_MODE          =   0x2000      @ Multi-play communication mode
SIO_UART_MODE           =   0x3000      @ UART communication mode

SIO_SCK_OUT             =   0x0000      @ Select external clock
SIO_SCK_IN              =   0x0001      @ Select internal clock
SIO_IN_SCK_256K         =   0x0000      @ Select internal clock 256KHz
SIO_IN_SCK_2M           =   0x0002      @                  Select 2MHz
SIO_ACK_RECV            =   0x0004      @ Request transfer
SIO_ACK_SEND            =   0x0008      @ Enable transfer
SIO_9600_BPS            =   0x0000      @ Baud rate 9600 bps
SIO_38400_BPS           =   0x0001      @          68400 bps
SIO_57600_BPS           =   0x0002      @          57600 bps
SIO_115200_BPS          =   0x0003      @        115200 bps
SIO_MULTI_CONNECT       =   0x0004      @ Multi-play communication connecting
SIO_MULTI_DISCONNECT    =   0x0000      @                  Disconnect
SIO_MULTI_PARENT        =   0x0008      @ Multi-play communication  Connect master
SIO_MULTI_CHILD         =   0x0000      @                  Connect slave
SIO_MULTI_SI            =   0x0004      @ Multi-play communication SI terminal
SIO_MULTI_SD            =   0x0008      @                  SD terminal
SIO_MULTI_BUSY          =   0x0080      @ Multi-play communicating
SIO_CTS_ENABLE          =   0x0004      @ Enable UART send enable signal
SIO_UART_7BIT           =   0x0000      @ UART communication data length 7 bit
SIO_UART_8BIT           =   0x0080      @                       8 bit
SIO_ERROR               =   0x0040      @ Detect error
SIO_START               =   0x0080      @ Start transfer
SIO_ENABLE              =   0x0080      @ Enable SIO
SIO_FIFO_ENABLE         =   0x0100      @ Enable FIFO
SIO_PARITY_ENABLE       =   0x0200      @ Enable parity
SIO_PARITY_EVEN         =   0x0000      @ Even parity
SIO_PARITY_ODD          =   0x0008      @ Odd parity
SIO_TRANS_ENABLE        =   0x0400      @ Enable transmitter
SIO_TRANS_DATA_FULL     =   0x0010      @ Transmitted data full
SIO_RECV_ENABLE         =   0x0800      @ Enable receiver
SIO_RECV_DATA_EMPTY     =   0x0020      @ No data received
SIO_IF_ENABLE           =   0x4000      @ Enable interrupt request

@--------------------------------------------------------------------
@-                      Joy Bus Control                             -
@--------------------------------------------------------------------

JOY_IF_RESET            =   0x01        @ Interrupt request of JOY bus reset
JOY_IF_RECV             =   0x02        @ Interrupt request of JOY bus receive completed
JOY_IF_SEND             =   0x04        @ Interrupt request of JOY bus send completed
JOY_IF_ENABLE           =   0x40        @ Enable interrupt request

@--------------------------------------------------------------------
@-                      JOY Bus Status                              -
@--------------------------------------------------------------------

JSTAT_FLAGS_MASK        =   0x30        @ General flag

JSTAT_FLAGS_SHIFT       =   4

JSTAT_RECV              =   0x02        @ Received status
JSTAT_SEND              =   0x08        @ Sent status

@--------------------------------------------------------------------
@-                      General Input/Output Control                -
@--------------------------------------------------------------------

R_SIO_MODE_MASTER_MASK  =   0xc000      @ SIO mode master

R_SIO_MODE_MASTER_SHIFT =   14

R_SIO_MASTER_MODE       =   0x0000      @ SIO master mode
R_DIRECT_MODE           =   0x8000      @ General input/output communication mode
R_JOY_MODE              =   0xc000      @ JOY communication mode

R_SC                    =   0x0001      @ data
R_SD                    =   0x0002
R_SI                    =   0x0004
R_SO                    =   0x0008
R_SC_I_O                =   0x0010      @ Select I/O
R_SD_I_O                =   0x0020
R_SI_I_O                =   0x0040
R_SO_I_O                =   0x0080
R_IF_ENABLE             =   0x0100      @ Enable interrupt request


@--------------------------------------------------------------------
@-                      DMA Control                                 -
@--------------------------------------------------------------------

DMA_TIMMING_MASK        =   0x30000000  @ Start Timing
DMA_COUNT_MASK          =   0x0000ffff  @ Transfer Number

DMA_TIMMING_SHIFT       =   30
DMA_COUNT_SHIFT         =   0

DMA_ENABLE              =   0x80000000  @ DMA Enable
DMA_IF_ENABLE           =   0x40000000  @ Interrupt Request Enable
DMA_TIMMING_IMM         =   0x00000000  @ Run Immediately
DMA_TIMMING_V_BLANK     =   0x10000000  @ Run V Blank
DMA_TIMMING_H_BLANK     =   0x20000000  @ Run H Blank
DMA_TIMMING_DISP        =   0x30000000  @ Run Display
DMA_TIMMING_SOUND       =   0x30000000  @ Run Sound FIFO Request
DMA_DREQ_ON             =   0x08000000  @ Data Request Synchronize Mode ON
DMA_16BIT_BUS           =   0x00000000  @ Select Bus Size 16Bit
DMA_32BIT_BUS           =   0x04000000  @ Select Bus Size 32Bit
DMA_CONTINUOUS_ON       =   0x02000000  @ Continuous Mode ON
DMA_SRC_INC             =   0x00000000  @ Select Source Increment
DMA_SRC_FIX             =   0x01000000  @ Select Source Fixed
DMA_DEST_INC            =   0x00000000  @ Select Destination Increment
DMA_DEST_RELOAD         =   0x00800000  @ Select Destination Increment/Reload

@--------------------------------------------------------------------
@-                      Timer Control                               -
@--------------------------------------------------------------------

TMR_PRESCALER_MASK      =   0x00030000  @ Prescaler Clock

TMR_PRESCALER_SHIFT     =   16

TMR_PRESCALER_1CK       =   0x00000000  @ Prescaler 1 Clock
TMR_PRESCALER_64CK      =   0x00010000  @           64 Clocks
TMR_PRESCALER_256CK     =   0x00020000  @         256 Clocks
TMR_PRESCALER_1024CK    =   0x00030000  @       1024 Clocks
TMR_IF_ENABLE           =   0x00400000  @ Interrupt Request Enable
TMR_ENABLE              =   0x00800000  @ Run Timer


@--------------------------------------------------------------------
@-                          Key                                     -
@--------------------------------------------------------------------

BUTTON_MASK             =   0x030f      @ Button
PLUS_KEY_MASK           =   0x00f0      @ Control Pad
ALL_KEY_MASK            =   0x03ff      @ All Keys

A_BUTTON                =   0x0001      @ A Button
B_BUTTON                =   0x0002      @ B Button
SELECT_BUTTON           =   0x0004      @ SELECT Button
START_BUTTON            =   0x0008      @ START Button
R_KEY                   =   0x0010      @ Right Button
L_KEY                   =   0x0020      @ Left Button
U_KEY                   =   0x0040      @ Up Button
D_KEY                   =   0x0080      @ Down Button
R_BUTTON                =   0x0100      @ R Button
L_BUTTON                =   0x0200      @ L Button
KEY_IF_ENABLE           =   0x4000      @ Interrupt Request Enable
KEY_OR_INTR             =   0x0000      @ Normal Key Interrupt
KEY_AND_INTR            =   0x8000      @ AND Key Interrupt


@--------------------------------------------------------------------
@-                  Game Pak Wait Control                           -
@--------------------------------------------------------------------

CST_SRAM_4WAIT          =   0x0000      @ SRAM 4 Wait
CST_SRAM_3WAIT          =   0x0001      @          3 Wait
CST_SRAM_2WAIT          =   0x0002      @          2 Wait
CST_SRAM_8WAIT          =   0x0003      @          8 Wait
CST_ROM0_1ST_4WAIT      =   0x0000      @ ROM 0 1st 4 Wait
CST_ROM0_1ST_3WAIT      =   0x0004      @            3 Wait
CST_ROM0_1ST_2WAIT      =   0x0008      @            2 Wait
CST_ROM0_1ST_8WAIT      =   0x000c      @            8 Wait
CST_ROM0_2ND_2WAIT      =   0x0000      @       2nd 2 Wait
CST_ROM0_2ND_1WAIT      =   0x0010      @            1 Wait
CST_ROM1_1ST_4WAIT      =   0x0000      @ ROM 1 1st 4 Wait
CST_ROM1_1ST_3WAIT      =   0x0020      @             3 Wait
CST_ROM1_1ST_2WAIT      =   0x0040      @             2 Wait
CST_ROM1_1ST_8WAIT      =   0x0060      @             8 Wait
CST_ROM1_2ND_4WAIT      =   0x0000      @       2nd 4 Wait
CST_ROM1_2ND_1WAIT      =   0x0080      @            1 Wait
CST_ROM2_1ST_4WAIT      =   0x0000      @ ROM 2 1st 4 Wait
CST_ROM2_1ST_3WAIT      =   0x0100      @             3 Wait
CST_ROM2_1ST_2WAIT      =   0x0200      @             2 Wait
CST_ROM2_1ST_8WAIT      =   0x0300      @             8 Wait
CST_ROM2_2ND_8WAIT      =   0x0000      @       2nd 8 Wait
CST_ROM2_2ND_1WAIT      =   0x0400      @              1 Wait

CST_PHI_OUT_NONE        =   0x0000      @ Phi Terminal Output Clock Lo Fixed
CST_PHI_OUT_4MCK        =   0x0800      @                4MHz
CST_PHI_OUT_8MCK        =   0x1000      @                8MHz
CST_PHI_OUT_16MCK       =   0x1800      @              16 MHz
CST_PREFETCH_ENABLE     =   0x4000      @ Prefetch Buffer Enable
CST_AGB                 =   0x0000      @ AGB Game Pak
CST_CGB                 =   0x8000      @ CGB Game Pak

@--------------------------------------------------------------------
@-                   System Call No. Defines                        -
@--------------------------------------------------------------------

SWI_NO_RESTART                  =   0       @ SoftReset()
SWI_NO_REGISTER_RAM_RESET       =   1       @ SoftReset()/RegisterRamReset()
SWI_NO_HALT                     =   2       @ Halt()
SWI_NO_STOP                     =   3       @ Stop()

SWI_NO_INTR_WAIT                =   4       @ IntrWait()
SWI_NO_VBLANK_INTR_WAIT         =   5       @ VBlankIntrWait()

SWI_NO_DIV                      =   6       @ Div()/DivRem()
SWI_NO_DIV_ARM                  =   7       @ DivArm()/DivRemArm()
SWI_NO_SQRT                     =   8       @ Sqrt()
SWI_NO_ARCTAN                   =   9       @ ArcTan()
SWI_NO_ARCTAN2                  =   10      @ ArcTan2()

SWI_NO_CPU_SET                  =   11      @ CpuSet()
SWI_NO_CPU_FAST_SET             =   12      @ CpuFastSet()
SWI_NO_BG_AFFINE_SET            =   14      @ BgAffineSet()
SWI_NO_OBJ_AFFINE_SET           =   15      @ ObjAffineSet()
SWI_NO_BIT_UNPACK               =   16      @ BitUnPack()
SWI_NO_LZ77_UNCOMP_WRAM         =   17      @ LZ77UnCompWram()
SWI_NO_LZ77_UNCOMP_VRAM         =   18      @ LZ77UnCompVram()
SWI_NO_HUFF_UNCOMP              =   19      @ HuffUnComp()
SWI_NO_RL_UNCOMP_WRAM           =   20      @ RLUnCompWram()
SWI_NO_RL_UNCOMP_VRAM           =   21      @ RLUnCompVram()
SWI_NO_DIFF_8BIT_UNFILTER_WRAM  =   22      @ Diff8bitUnFilterWram()
SWI_NO_DIFF_8BIT_UNFILTER_VRAM  =   23      @ Diff8bitUnFilterVram()
SWI_NO_DIFF_16BIT_UNFILTER      =   24      @ Diff16bitUnFilter()

SWI_NO_SOUND_BIAS_CHANGE        =   25      @ SoundBiasSet()/SoundBiasReset()
SWI_NO_SOUND_DRIVER_INIT        =   26      @ SoundDriverInit()
SWI_NO_SOUND_DRIVER_MODE        =   27      @ SoundDriverMode()
SWI_NO_SOUND_DRIVER_MAIN        =   28      @ SoundDriverMain()
SWI_NO_SOUND_DRIVER_VSYNC       =   29      @ SoundDriverVSync()
SWI_NO_SOUND_DRIVER_VSYNC_OFF   =   40      @ SoundDriverVSyncOff()
SWI_NO_SOUND_DRIVER_VSYNC_ON    =   41      @ SoundDriverVSyncOn()
SWI_NO_SOUND_CHANNEL_CLEAR      =   30      @ SoundChannelClear()
SWI_NO_MIDI_KEY_2_FREQ          =   31      @ MidiKey2Freq()
SWI_NO_MUSIC_PLAYER_OPEN        =   32      @ MusicPlayerOpen()
SWI_NO_MUSIC_PLAYER_START       =   33      @ MusicPlayerStart()
SWI_NO_MUSIC_PLAYER_STOP        =   34      @ MusicPlayerStop()
SWI_NO_MUSIC_PLAYER_CONTINUE    =   35      @ MusicPlayerContinue()
SWI_NO_MUSIC_PLAYER_FADEOUT     =   36      @ MusicPlayerFadeOut()

SWI_NO_MULTI_BOOT               =   37      @ MultiBoot()

@--------------------------------------------------------------------
@-                      System Call Macros                                -
@--------------------------------------------------------------------

    .MACRO    SystemCall16 No
        swi \No
    .ENDM
    .MACRO    SystemCall32 No
        swi \No << 16
    .ENDM

    .MACRO    Halt                Bits    @ HALT
        SystemCall\Bits 2
    .ENDM
    .MACRO    Stop                Bits    @ STOP
        SystemCall\Bits 3
    .ENDM

@--------------------------------------------------------------------
@-                   Software Reset                                -
@--------------------------------------------------------------------

    .EXTERN SoftReset

@ * Reset the register and RAM that were specified by ResetFlags and return to address 0x8000000.
@ * The area between 0x3007e00 and 0x3007fff of CPU core register and CPU internal RAM is forced to be cleared.
@
@ * Arguments:
@    r0:                  Specify the register and RAM that are to be reset (See AgbDefine.s for reference)

@--------------------------------------------------------------------
@-               Reset Register & RAM                          -
@--------------------------------------------------------------------

    .EXTERN RegisterRamReset

@ * Reset the register and RAM that were specified by ResetFlags.
@ * However, the area between 0x3007e00~0x3007fff of CPU internal RAM will not be cleared.
@
@ * Arguments:
@    r0:                  Specify the register and RAM that are to be reset (See AgbDefine.s for reference)

@--------------------------------------------------------------------
@-                   HALT                                       -
@--------------------------------------------------------------------

@   .EXTERN Halt

@ * It halts CPU only.
@ * Applicable interrupt is enabled (set in IE).
@  Return by interrupt request (set in IF).

@--------------------------------------------------------------------
@-                   STOP                                       -
@--------------------------------------------------------------------

    .EXTERN SoundBiasSet
    .EXTERN SoundBiasReset

@   .EXTERN Stop

@ * Stop source oscillation.
@ * When applicable interrupt is enabled (set in IE), return with the generation of the conditions
@  for an interrupt request from a key, a game pak or SIO.
@ * IF flag will not be set since souce oscillation is stopped.
@ * LCDC must be OFF when executed.

@--------------------------------------------------------------------
@-               Interrupt Wait                                   -
@--------------------------------------------------------------------

    .EXTERN IntrWait

@ * Continue to wait in the Halt status until the interrupt specified by IntrFlags generates.
@ * Set a flag that corresponds to INTR_CHECK_BUF(0x3007ff8) by interrupt process.
@ * Overhead that calls for system call can be reduced by using multiple interrupts simultaneously when compared
@  with the case where Halt() is called back repeatedly.
@
@ * Arguments:
@    r0:                 Specify whether or not to clear when applicable flag has already been set.
@    r1:                 Specify an interrupt that waits. (See AgbDefine.s for reference)

@--------------------------------------------------------------------
@-               V-blank Interrupt Wait                         -
@--------------------------------------------------------------------

    .EXTERN VBlankIntrWait

@ * Continue to wait in Halt status until V-blank interrupt generates.
@ * Set a flag that corresponds to INTR_CHECK_BUF(0x3007ff8) in interrupt process.
@ * Overhead that calls for system call can be reduced by using multiple interrupts simultaneously when compared
@  with the case where the Halt() is called back repeatedly.
@ * It is equivalent to IntrWait(1, V_BLANK_INTR_FLAG).

@--------------------------------------------------------------------
@-                   Signed Division (Quoitient)                           -
@--------------------------------------------------------------------

    .EXTERN Div                 @ Compatible with the library by RedHat (Formerly Cygnus)
    .EXTERN DivArm              @ Compatible with the library by ARM

@ * Calculate Number/Denom.
@ * The register value returns by r0=Number/Denom, r1=Number%Denom,
@  r3=|Number/Denom|.
@
@ * Arguments: (Reversal for the library by ARM)
@    r0:                 Numerator
@    r1:                 Denominator

@--------------------------------------------------------------------
@-                   Signed Division (Remainder)                         -
@--------------------------------------------------------------------

    .EXTERN DivRem              @ Compatible with the library by RedHat (Formerly Cygnus)
    .EXTERN DivRemArm           @ Compatible with the library by ARM

@ * Calculate Number%Denom.
@ * The register value returns by r0=Number%Denom, r1=Number%Denom,
@  r3=|Number/Denom|.
@
@ * Arguments: (Reversal for the library by ARM)
@    r0:                 Numerator
@    r1:                 Denominator

@--------------------------------------------------------------------
@-                   Square Root                                         -
@--------------------------------------------------------------------

    .EXTERN Sqrt

@ * Calculate square root.
@ * In order to make it accurate, shift argumentX by the multiple of 2 to the left, and shift return value to the right
@  by the multiplied number used in order to match digit.

@--------------------------------------------------------------------
@-                   Arc Tangent                                         -
@--------------------------------------------------------------------

    .EXTERN ArcTan

@ * Calculate arc tangent value.
@ * For return value, return -pi/2Ɓpi/2 within the range of 0xc000~0x4000.
@ * However, the range Ɓ-pi/4,pi/4 has a problem with accuracy.
@
@ * Arguments:
@    r0:
@        Sign: 1
@        Integer part: 1
@        Decimal part: 14

@--------------------------------------------------------------------
@-                   Arc Tangent 2                                       -
@--------------------------------------------------------------------

    .EXTERN ArcTan2

@ * Calculate the arc tangent value after correction process.
@ * Use this for regular purpose.
@ * For return value, return 0Ɓ2pi within the range of 0~0xffff.
@
@ * Arguments:
@    r0, r1:
@        Sign: 1
@        Integer part: 1
@        Decimal part: 14

@--------------------------------------------------------------------
@-                   Set CPU                                   -
@--------------------------------------------------------------------

    .EXTERN CpuSet

@ * Clear or copy RAM by using parameter that is compatible with DmaSet macro.
@ * RAM is forcefully accessed at every 4 byte boundary by 32bit transfer, and it is to be accessed at every 2 byte boundary by 16bit transfer.
@
@
@ * Arguments:
@    r0:                 Source address
@    r1:                 Destination address
@    r2:                 Only DMA_SRC_FIX/DMA_32BIT_BUS/DMA_COUNT_MASK are valid
@          DMA_SRC_FIX(  0, 1) = (Source address increment, Fixed source address)
@          DMA_32BIT_BUS(0, 1) = (16bit transfer, 32bit transfer)
@          DMA_COUNT_MASK & DmaCntData = Number of times transferred
@
@ * Upper macro:
@  CpuClear, CpuArrayClear, CpuCopy, CpuArrayCopy

@--------------------------------------------------------------------
@-                   High-speed Set CPU                               -
@--------------------------------------------------------------------

    .EXTERN CpuFastSet

@ * Clear or copy RAM in high speed by using parameter that is compatible with DmaSet macro.
@ * However, it is a 32bit transfer per 32 bytes (8 words).
@ * Even if an argument other than 4byte boundary is given, it is accessed forcefully at every 4 byte boundary.
@
@ * Arguments:
@    r0:                 Source address
@    r1:                 Destination address
@    r2:                 Only DMA_SRC_FIX/DMA_COUNT_MASK are valid
@          DMA_SRC_FIX(0, 1) = (Source address increment, Fixed source address)
@          DMA_COUNT_MASK & DmaCntData = Number of times transferred
@
@ * Upper macro:
@  CpuFastClear, CpuFastArrayClear, CpuFastCopy, CpuFastArrayCopy

@--------------------------------------------------------------------
@-                   Set BG Affine Transformation                        -
@--------------------------------------------------------------------

    .EXTERN BgAffineSet

@ * Calculate and set BG affine parameter.
@
@ * Arguments:
@    r0:                       Source address
@    r1:                       Destination address
@    r2:                       Number of times calculated
@
@ * BgAffineSrcData Structure
@    s32 SrcCenterX            Source data center coordinate (8bits to the right of decimal point)
@    s32 SrcCenterY
@    s16 DispCenterX           Display center coordinate
@    s16 DispCenterY
@    s16 RatioX                Scaling ratio (8bits to the right of decimal point)
@    s16 RatioY
@    u16 Theta                 Rotation angle (8bits to the right of decimal point)
@                                  Valid range    0 - 0xffff
@ * BgAffineDestData Structure
@    s16 H_DiffX               Line direction X coordinate difference
@    s16 V_DiffX                 Vertical direction X coordinate difference
@    s16 H_DiffY               Line direction Y coordinate difference
@    s16 V_DiffY                 Vertical direction Y coordinate difference
@    s32 StartX                Start X coordinate
@    s32 StartY                Start Y coordinate

@--------------------------------------------------------------------
@-                   Set OBJ Affine Transformation                      -
@--------------------------------------------------------------------

    .EXTERN ObjAffineSet

@ * Calculate and set affine parameter of OBJ.
@
@ * Arguments:
@    r0:                       Source address
@    r1:                       Destination address
@    r2:                       Number of times calculated
@    r3:                       Offset byte size of parameter address (normally 2|8)
@                              Designate "8" when directly setting to OAM
@
@ * ObjAffineSrcData Structure
@    s16 RatioX                Scaling ratio (8bits to the right of decimal point)
@    s16 RatioY
@    u16 Theta                 Rotation angle (8bits to the right of decimal point)
@                                  Valid range    0 - 0xffff
@ * ObjAffineDestData Structure
@    s16 H_DiffX               Line direction X coordinate difference
@    s16 V_DiffX                 Vertical direction X coordinate difference
@    s16 H_DiffY               Line direction Y coordinate difference
@    s16 V_DiffY                 Vertical direction Y coordinate difference

@--------------------------------------------------------------------
@-                       Expand Bit Compressed Data                          -
@--------------------------------------------------------------------

    .EXTERN BitUnPack

@ * Expand data filled with bits fixed 0.
@ * Match destination address to 4 byte boundary.
@
@ * Arguments:
@    r0:                     Source address
@    r1:                     Destination address
@    r2:                     Address of BitUnPackParam structure data
@
@ * BitUnPackParam Structure
@    u16 SrcNum              Source data byte size
@    u8  SrcBitNum           1 source data bit size
@    u8  DestBitNum          1 destination data bit size
@    u32 DestOffset:31       Offset number to add to source data
@        DestOffset0_On:1    Flag for adding/not adding offset to 0 data

@--------------------------------------------------------------------
@-                   LZ77 Compressed Data Expansion (-> Work RAM)      -
@--------------------------------------------------------------------

    .EXTERN LZ77UnCompWram

@ * Expand LZ77 compressed data and write by 8bits.
@ * Cannot expand directly to VRAM.
@ * When compressed data size is not a multiple of 4, fill with 0 to adjust.
@ * Match source address to 4 byte boundary.
@
@ * Arguments:
@    r0:                     Source address
@    r1:                     Destination address
@
@ * Data header
@    u32 :4                  Reserve
@        CompType:4          Compressed type (= 1)
@        DestSize:24         Post-expansion data size
@
@ * Flag data format
@    u8  Flags               Compressed/non-compressed flag
@                            (0, 1) = (Non-compressed data, Compressed data)
@ * Code data format (Big Endian)
@    u16 Length:4            Expansion data length  - 3 (Compress only when matching length is 3 bytes or more)
@        Offset:12           Matching data offset - 1

@--------------------------------------------------------------------
@-                   Expand LZ77 Compressed Data  (->VRAM)           -
@--------------------------------------------------------------------

    .EXTERN LZ77UnCompVram

@ * Expand LZ77 compressed data and write by 16bits.
@ * Can be expanded to work RAM, however, it is slower than LZ77UnCompWram().
@ * Specify the compressed data in which matching character array was retrieved by 2 bytes before.
@ * When compressed data size is not a multiple of 4, fill with 0 to adjust.
@ * Match source address to 4 byte boundary.
@
@ * Arguments:
@    r0:                     Source address
@    r1:                     Destination address
@
@ * Data header
@    u32 :4                  Reserve
@        CompType:4          Compressed type (= 1)
@        DestSize:24         Post-expansion data size
@
@ * Flag data format
@    u8  Flags               Compressed/non-compressed flag
@                            (0, 1) = (Non-compressed data, Compressed data)
@ * Code data format (Big Endian)
@    u16 Length:4            Expansion data length - 3 (Compress only when matching length is 3 bytes or more)
@        Offset:12           Matching data offset ( >= 2) - 1

@--------------------------------------------------------------------
@-                   Expand Huffman Compressed Data                         -
@--------------------------------------------------------------------

    .EXTERN HuffUnComp

@ * Expand Huffman comrpessed data and write by 32 bits.
@ * When compressed data size is not a multiple of 4, fill with 0 to adjust.
@ * Match source address to 4 byte boundary.
@
@ * Arguments:
@    r0:                     Source address
@    r1:                     Destination address
@
@ * Data header
@    u32 BitSize:4           1 data bit size (normally 4|8)
@        CompType:4          Compressed type (= 2)
@        DestSize:24         Post-expansion data size
@
@ * Tree table
@    u8           TreeSize        Tree table size/2 - 1
@    TreeNodeData RootNode        Root node
@
@    TreeNodeData LeftNode        Root left node
@    TreeNodeData RightNode       Root right node
@
@    TreeNodeData LeftleftNode    Left left node
@    TreeNodeData leftRightNode   Left right node
@
@    TreeNodeData RightLeftNode   Right left node
@    TreeNodeData RightRightNode  Right right node
@
@            .
@            .
@
@  Following is the main body of compressed data
@
@ * TreeNodeData Structure
@    u8  NextNodeOffset:6    Offset to the next node data  - 1 (2Byte unit)
@        RightEndFlag:1      Right node end flag
@        LeftEndFlag:1       Left node end flag
@                            When end flag is set
@                            Data is in the next node

@--------------------------------------------------------------------
@-                   Expand Run Length Compressed Data  (-> Work RAM)   -
@--------------------------------------------------------------------

    .EXTERN RLUnCompWram

@ * Expand run length compressed data and write by 8 bits.
@ * Cannot directly expand to VRAM.
@ * When compressed data size is not a multiple of 4, fill with 0 to adjust.
@ * Match source address to 4 byte boundary.
@
@ * Arguments:
@    r0:                     Source address
@    r1:                     Destination address
@
@ * Data header
@    u32 :4                  Reserve
@        CompType:4          Compressed data  ( = 3)
@        DestSize:24         Post-expansion data size
@
@ * Flag data format
@    u8  Length:7            Expansion data length - 1 (when not compressed)
@                            Expansion data length - 3 (Compress only when continuous length is 3 bytes or more)
@        Flag:1               (0, 1) =  (Non-compressed data, Compressed data)

@--------------------------------------------------------------------
@-                   Expand Run Length Compressed Data (->VRAM)     -
@--------------------------------------------------------------------

    .EXTERN RLUnCompVram

@ * Expand expand run length compressed data and write by 16 bits.
@ * It can expand to work RAM, however, it is slower than RLUnCompWram().
@ * When compressed data size is not a multiple of 4, fill with 0 to adjust.
@ * Match source address to 4 byte boundary.
@
@ * Arguments:
@    r0:                     Source address
@    r1:                     Destination address
@
@ * Data header
@    u32 :4                  Reserve
@        CompType:4          Compressed type (= 3)
@        DestSize:24         Post-expansion data size
@
@ * Flag data format
@    u8  Length:7            Expansion data length - 1 (when not compressed)
@                            Expand data length - 3 (compress only when the consecutive length is 3 bytes or more)
@        Flag:1               (0, 1) =  (Non-compressed data, Compressed data)

@--------------------------------------------------------------------
@-                   Expand Difference Filter (8bits -> Work RAM)         -
@--------------------------------------------------------------------

    .EXTERN Diff8bitUnFilterWram

@ * Expand 8-bit difference filter manipulation data and write by 8 bits.
@ * Cannot expand directly to VRAM.
@ * Match source address to 4 byte boundary.
@
@ * Arguments:
@    r0:                     Source address
@    r1:                     Destination address
@
@ * Data header
@    u32 ByteSize:4          1 data byte size ( = 1)
@        FiltType:4          Filter type ( = 8)
@        DestSize:24         Post-expansion data size
@
@ * Data format
@    u8  Origin              Source data
@    u8  Diff                Difference data
@         .
@         .

@--------------------------------------------------------------------
@-                   Expand Difference Filter  (8 bits -> VRAM)             -
@--------------------------------------------------------------------

    .EXTERN Diff8bitUnFilterVram

@ * Expand 8-bit difference filter manipulation data and write by 16 bits.
@ * Can expand to work RAM, however, it is slower than Diff8bitUnFilterVram().
@ * Match source address to 4 byte boundary.
@
@ * Arguments:
@    r0:                     Source address
@    r1:                     Destination address
@
@ * Data header
@    u32 ByteSize:4          1 data byte size ( = 1)
@        FilterType:4        Filter type ( = 8)
@        DestSize:24         Post-expansion data size
@
@ * Data format
@    u8  Origin              Source data
@    u8  Diff                Difference data
@         .
@         .

@--------------------------------------------------------------------
@-                   Expand difference filter  (16 bits)             -
@--------------------------------------------------------------------

    .EXTERN Diff16bitUnFilter

@ * Expand 16-bit difference filter manipulation data and write by 16 bits.
@ * Match source address by 4 byte boundary.
@
@ * Arguments:
@    r0:                     Source address
@    r1:                     Destination address
@
@ * Data header
@    u32 ByteSize:4          1 data byte size ( = 2)
@        FilterType:4        Filter type ( = 8)
@        DestSize:24         Post-expansion data size
@
@ * Data format
@    u16 Origin              Source data
@    u16 Diff                Difference data
@         .
@         .

@--------------------------------------------------------------------
@-              Set Sound BIAS                             -
@--------------------------------------------------------------------

    .EXTERN SoundBiasSet

@ * Move sound BIAS from 0 to intermediate value  (0x200).

@--------------------------------------------------------------------
@-              Reset Sound BIAS                           -
@--------------------------------------------------------------------

    .EXTERN SoundBiasReset

@ * Move sound BIAS from 0 to intermediate value (0x200).

@--------------------------------------------------------------------
@-              Initialize Sound Driver                             -
@--------------------------------------------------------------------

    .EXTERN SoundDriverInit

@  Initialize sound driver.
@  Call just once when game starts.
@  At that time, it is necessary to obtain the work area "sa" beforehand.  Even if the work area is prepared separately,
@  it is not possible to execute the main driver multiple times simultaneously.

@--------------------------------------------------------------------
@-              Sound Driver Move Mode                         -
@--------------------------------------------------------------------

    .EXTERN SoundDriverMode

@  Set move mode of sound driver.
@
@  | Reverb of direct sound (Initial value 0)
@      mode = SOUND_MODE_REVERB_SET + (Reverb value 0~127);
@
@  | Simultaneous sound number of direct sound (Initial value 8)
@      mode = (Simultaneous sound number 1~12) << SOUND_MODE_MAXCHN_SHIFT;
@
@  | Master volume of direct sound (Initial value 15)
@      mode = (Volume 1~15) << SOUND_MODE_MASVOL_SHIFT;
@
@  | Play frequency of direct sound (Initial value 13379Hz)
@      mode = SOUND_MODE_FREQ_?????;
@             -> a value that is defined in AgbSound.h (12 types)
@
@  | Bit number of Final D/A converter (Initial value 8bits)
@      mode = SOUND_MODE_DA_BIT_?;
@                            (up) 9~6
@
@  | Above value can be set simultaneously by conducting OR.

@--------------------------------------------------------------------
@-              Sound Driver Main                             -
@--------------------------------------------------------------------

    .EXTERN SoundDriverMain

@  Main of sound driver.
@  Call every 1/60 second.  The process flow is to call SoundDriverVSync(), which is stated below,
@  immediately after V-blank interrupt, then to call main routine after processing BG adn OBJ.

@--------------------------------------------------------------------
@-              Reset Sound Driver DMA                     -
@--------------------------------------------------------------------

    .EXTERN SoundDriverVSync

@  Very short system call that resets sound DMA.
@  As timing is very severe, call IMMEDIATELY AFTER V-blank interrupt, which is called
@  every 1/60 second, is in.

@--------------------------------------------------------------------
@-              Clear Sound Channel                           -
@--------------------------------------------------------------------

    .EXTERN SoundChannelClear

@  Clear all the channels of direct sound and stop sound.
@  This function may not operate properly when the library that expands the function of
@  sound driver is installed.  Do not use if such malfunction occurs.

@--------------------------------------------------------------------
@-                 Stop Sound DMA                             -
@--------------------------------------------------------------------

    .EXTERN SoundDriverVSyncOff

@  When SoundDriverVSync() cannot be called every 1/60 second because V-blank interrupt is
@  stopped due to the status of the main program, etc., it is necessary to stop sound DMA by
@  using this function.  Otherwise, DMA will not stop even if it goes over the buffer, which will cause
@  terrible noise.

@--------------------------------------------------------------------
@-                 Re-opening of Sound DMA                             -
@--------------------------------------------------------------------

    .EXTERN SoundDriverVSyncOn

@  This is a function that re-opens the sound DMA which is stopped by SoundDriverVSyncOff()
@  described above.  Set it so that V-blank interrupt is entered and SoundDriverVSync() is called
@  within 2/60 seconds after this function is called.

@--------------------------------------------------------------------
@-               Calculate Interval by MIDI Key                        -
@--------------------------------------------------------------------

    .EXTERN MidiKey2Freq

@  When playing the waveform data "wa" by the interval (MIDI KEY) mk or fine adjustment value fp,
@  it calculates the value that is assigned to ((SoundArea)sa).vchn[x].fr.

@--------------------------------------------------------------------
@-            Open Music Player                      -
@--------------------------------------------------------------------

    .EXTERN MusicPlayerOpen

@  Open music player.
@  "ma" is a work area, "ta" is a work area by track, and "tn" is the number of track (maximum 16).
@  If "ma and, ta[tn]" are prepared separately, multiple players can be opened, which enables playing multiple
@  songs simultaneously.  For example, you can open one music player for BGM and four music players for SE.
@
@  *Music player function gruop is the upper function of sound driver function group.

@--------------------------------------------------------------------
@-                       Start Playing Songs                             -
@--------------------------------------------------------------------

    .EXTERN MusicPlayerStart

@  Play a song "so" by using the music player of the work area "ma".

@--------------------------------------------------------------------
@-                     Pause a song that is played                       -
@--------------------------------------------------------------------

    .EXTERN MusicPlayerStop

@  Pause the song that is currently played by the music player of the work area "ma".

@--------------------------------------------------------------------
@-                        Re-open the song that is paused.                            -
@--------------------------------------------------------------------

    .EXTERN MusicPlayerContinue

@  Re-open the song that is paused in the music player of the work area "ma".

@--------------------------------------------------------------------
@-                       Fade out a song                         -
@--------------------------------------------------------------------

    .EXTERN MusicPlayerFadeOut

@  Fade out and stop teh song that is currently played in the work area "ma" by the speed "sp".


@--------------------------------------------------------------------
@-                   Multi-play boot                             -
@--------------------------------------------------------------------

    .EXTERN MultiBoot

@ * Main process for the server of multi-play boot.
@ * The specified recognition process must be conducted in advance between all the client AGBs.

@--------------------------------------------------------------------
@-                      Memory Map                                  -
@--------------------------------------------------------------------

BOOT_ROM                =   0x00000000          	@ Boot ROM
BOOT_ROM_END            =   (BOOT_ROM + 0x4000)

EX_WRAM                 =   0x02000000          	@ CPU External Work RAM
EX_WRAM_END             =   (EX_WRAM + 0x40000)

CPU_WRAM                =   0x03000000          	@ CPU Internal Work RAM
CPU_WRAM_END            =   (CPU_WRAM + 0x8000)
WRAM                    =    EX_WRAM            	@ Entire Work RAM
WRAM_END                =    CPU_WRAM_END
SOUND_AREA_ADDR_BUF     =   (CPU_WRAM_END - 0x10)	@ Sound Driver Work Address
INTR_CHECK_BUF          =   (CPU_WRAM_END - 0x8)	@ Interrupt Check
INTR_VECTOR_BUF         =   (CPU_WRAM_END - 0x4)	@ Interrupt Branch Address

REG_BASE                =   0x04000000          	@ Registers
REG_END                 =   (REG_BASE +  0x300)

PLTT                    =   0x05000000          	@ Palette RAM
PLTT_END                =   (PLTT +      0x400)
BG_PLTT                 =   (PLTT +        0x0) 	@ BG Palette RAM
BG_PLTT_END             =   (PLTT +      0x200)
OBJ_PLTT                =   (PLTT +      0x200) 	@ OBJ Palette RAM
OBJ_PLTT_END            =   (PLTT +      0x400)

VRAM                    =   0x06000000          @ VRAM
VRAM_END                =   (VRAM +    0x18000)
BG_VRAM                 =   (VRAM +        0x0) @ BG Character/Screen RAM
BG_BITMAP0_VRAM         =   (VRAM +        0x0) @ BG Bitmap 0 RAM
BG_BITMAP1_VRAM         =   (VRAM +     0xa000) @ BG Bitmap 1 RAM
OBJ_MODE0_VRAM          =   (VRAM +    0x10000) @ OBJ Character RAM
OBJ_MODE1_VRAM          =   (VRAM +    0x10000)
OBJ_MODE2_VRAM          =   (VRAM +    0x10000)
OBJ_MODE3_VRAM          =   (VRAM +    0x14000)
OBJ_MODE4_VRAM          =   (VRAM +    0x14000)
OBJ_MODE5_VRAM          =   (VRAM +    0x14000)
OBJ_VRAM_END            =   (VRAM +    0x18000)

OAM                     =   0x07000000          @ OAM
OAM_END                 =   (OAM +       0x400)

ROM_BANK0               =   0x08000000          @ ROM Bank 0
ROM_BANK0_END           =   0x0a000000
ROM_BANK1               =   0x0a000000          @ ROM Bank 1
ROM_BANK1_END           =   0x0c000000
ROM_BANK2               =   0x0c000000          @ ROM Bank 2
ROM_BANK2_END           =   0x0e000000
RAM_BANK                =   0x0c000000          @ RAM Bank
RAM_BANK_END            =   0x10000000

FLASH_1M                =   0x09fe0000          @ 1M Flash Memory
FLASH_1M_END            =   0x0a000000

@--------------------------------------------------------------------
@-                      Memory Size                                 -
@--------------------------------------------------------------------

BOOT_ROM_SIZE           =   0x4000              @ Boot ROM
EX_WRAM_SIZE            =   0x40000             @ CPU External Work RAM
CPU_WRAM_SIZE           =   0x8000              @ CPU Internal Work RAM
WRAM_SIZE               =   0x48000             @ Entire Work RAM
USR_BUF_SIZE            =   0x47fa0             @ User Area
SYS_BUF_SIZE            =   0x80                @ System Area
PLTT_SIZE               =   (2*256*2)           @ Palette RAM
BG_PLTT_SIZE            =   (2*256)             @ BG Palette RAM
OBJ_PLTT_SIZE           =   (2*256)             @ OBJ Palette RAM
VRAM_SIZE               =   0x18000             @ VRAM
BG_MODE0_VRAM_SIZE      =   0x10000             @ BG Character/Screen
BG_MODE1_VRAM_SIZE      =   0x10000
BG_MODE2_VRAM_SIZE      =   0x10000
BG_MODE3_VRAM_SIZE      =   0x14000             @ BG Bitmap
BG_MODE4_VRAM_SIZE      =   0x0a000
BG_MODE5_VRAM_SIZE      =   0x0a000
OBJ_MODE0_VRAM_SIZE     =   0x08000             @ OBJ Character
OBJ_MODE1_VRAM_SIZE     =   0x08000
OBJ_MODE2_VRAM_SIZE     =   0x08000
OBJ_MODE3_VRAM_SIZE     =   0x04000
OBJ_MODE4_VRAM_SIZE     =   0x04000
OBJ_MODE5_VRAM_SIZE     =   0x04000
OAM_SIZE                =   (8*128)             @ OAM

ROM_BANK_SIZE           =   0x02000000          @ ROM Bank
ROM_BANK0_SIZE          =   0x02000000          @ ROM Bank 0
ROM_BANK1_SIZE          =   0x02000000          @ ROM Bank 1
ROM_BANK2_SIZE          =   0x02000000          @ ROM Bank 2
RAM_BANK_SIZE           =   0x02000000          @ RAM Bank

FLASH_1M_SIZE           =   0x20000             @ 1M Flash Memory

@--------------------------------------------------------------------
@-                      Register Address                            -
@--------------------------------------------------------------------

REG_PAUSE       =   (REG_BASE + 0x300)  @ PAUSE

REG_IME         =   (REG_BASE + 0x208)  @ Interrupt Master Enable
REG_IE          =   (REG_BASE + 0x200)  @ Interrupt Enable
REG_IF          =   (REG_BASE + 0x202)  @ Interrupt Request

REG_WAITCNT     =   (REG_BASE + 0x204)  @ Game Pak Wait Control

REG_DISPCNT     =   (REG_BASE + 0x0)    @ Display Control

REG_STAT        =   (REG_BASE + 0x4)    @ Status
REG_VCOUNT      =   (REG_BASE + 0x6)    @ V Counter

REG_MOSAIC      =   (REG_BASE + 0x4c)   @ Mosaic Size

REG_BLDCNT      =   (REG_BASE + 0x50)   @ Blend Mode Control
REG_BLDMOD      =   REG_BLDCNT
REG_BLDALPHA    =   (REG_BASE + 0x52)   @ Semi-transparent Parameter
REG_BLDY        =   (REG_BASE + 0x54)   @ Brightness Change Parameter

REG_WINCNT      =   (REG_BASE + 0x40)   @ Window Control
REG_WIN0H       =   (REG_BASE + 0x40)   @ Window 0 Horizontal Area
REG_WIN1H       =   (REG_BASE + 0x42)   @ Window 1 Horizontal Area
REG_WIN0V       =   (REG_BASE + 0x44)   @ Window 0 Vertical Area
REG_WIN1V       =   (REG_BASE + 0x46)   @ Window 1 Vertical Area
REG_WININ       =   (REG_BASE + 0x48)   @ Internal Window Control
REG_WIN0        =   (REG_BASE + 0x48)   @ Window 0 Control
REG_WIN1        =   (REG_BASE + 0x49)   @ Window 1 Control
REG_WINOUT      =   (REG_BASE + 0x4a)   @ External Window Control
REG_OBJWIN      =   (REG_BASE + 0x4b)   @ OBJ Window Control


REG_BGCNT       =   (REG_BASE + 0x8)    @ BG Control
REG_BG0CNT      =   (REG_BASE + 0x8)    @ BG 0 Control
REG_BG1CNT      =   (REG_BASE + 0xa)    @ BG 1 Control
REG_BG2CNT      =   (REG_BASE + 0xc)    @ BG 2 Control
REG_BG3CNT      =   (REG_BASE + 0xe)    @ BG 3 Control
REG_BGOFS       =   (REG_BASE + 0x10)   @ BG Offset
REG_BG0HOFS     =   (REG_BASE + 0x10)   @ BG 0 H Offset
REG_BG0VOFS     =   (REG_BASE + 0x12)   @ BG 0 V Offset
REG_BG1HOFS     =   (REG_BASE + 0x14)   @ BG 1 H Offset
REG_BG1VOFS     =   (REG_BASE + 0x16)   @ BG 1 V Offset
REG_BG2HOFS     =   (REG_BASE + 0x18)   @ BG 2 H Offset
REG_BG2VOFS     =   (REG_BASE + 0x1a)   @ BG 2 V Offset
REG_BG3HOFS     =   (REG_BASE + 0x1c)   @ BG 3 H Offset
REG_BG3VOFS     =   (REG_BASE + 0x1e)   @ BG 3 V Offset
REG_BG2AFFINE   =   (REG_BASE + 0x20)   @ BG 2 Affine Transformation Parameters
REG_BG2PA       =   (REG_BASE + 0x20)   @ BG 2 Line Direction X Coordinate Differernce
REG_BG2PB       =   (REG_BASE + 0x22)   @ BG 2 Vertical Direction X Coordinate Difference
REG_BG2PC       =   (REG_BASE + 0x24)   @ BG 2 Line Direction Y Coordinate Difference
REG_BG2PD       =   (REG_BASE + 0x26)   @ BG 2 Vertical Direction Y Coordinate Difference
REG_BG2X        =   (REG_BASE + 0x28)   @ BG 2 Start X Coordinate
REG_BG2X_L      =   (REG_BASE + 0x28)
REG_BG2X_H      =   (REG_BASE + 0x2a)
REG_BG2Y        =   (REG_BASE + 0x2c)   @ BG 2 Start Y Coordinate
REG_BG2Y_L      =   (REG_BASE + 0x2c)
REG_BG2Y_H      =   (REG_BASE + 0x2e)
REG_BG3AFFINE   =   (REG_BASE + 0x30)   @ BG 3 Affine Transformation Parameters
REG_BG3PA       =   (REG_BASE + 0x30)   @ BG 3 Line Direction X Coordinate Difference
REG_BG3PB       =   (REG_BASE + 0x32)   @ BG 3 Vertical Direction X Coordinate Difference
REG_BG3PC       =   (REG_BASE + 0x34)   @ BG 3 Line Direction Y Coordinate Difference
REG_BG3PD       =   (REG_BASE + 0x36)   @ BG 3 Vertical Direction Y Coordinate Difference
REG_BG3X        =   (REG_BASE + 0x38)   @ BG 3 Start X Coordinate
REG_BG3X_L      =   (REG_BASE + 0x38)
REG_BG3X_H      =   (REG_BASE + 0x3a)
REG_BG3Y        =   (REG_BASE + 0x3c)   @ BG 3 Start Y Coordinate
REG_BG3Y_L      =   (REG_BASE + 0x3c)
REG_BG3Y_H      =   (REG_BASE + 0x3e)


REG_SOUNDCNT    =   (REG_BASE + 0x80)   @ Sound Control
REG_SOUNDCNT_L  =   (REG_BASE + 0x80)
REG_SOUNDCNT_H  =   (REG_BASE + 0x82)
REG_SOUNDCNT_X  =   (REG_BASE + 0x84)

REG_SOUNDBIAS   =   (REG_BASE + 0x88)   @ Sound BIAS

REG_SOUND1CNT   =   (REG_BASE + 0x60)   @ Sound 1 Control
REG_SOUND1CNT_L =   (REG_BASE + 0x60)
REG_SOUND1CNT_H =   (REG_BASE + 0x62)
REG_SOUND1CNT_X =   (REG_BASE + 0x64)
REG_SOUND2CNT   =   (REG_BASE + 0x68)   @ Sound 2 Control
REG_SOUND2CNT_L =   (REG_BASE + 0x68)
REG_SOUND2CNT_H =   (REG_BASE + 0x6c)
REG_SOUND3CNT   =   (REG_BASE + 0x70)   @ Sound 3 Control
REG_SOUND3CNT_L =   (REG_BASE + 0x70)
REG_SOUND3CNT_H =   (REG_BASE + 0x72)
REG_SOUND3CNT_X =   (REG_BASE + 0x74)
REG_SOUND4CNT   =   (REG_BASE + 0x78)   @ Sound 4 Control
REG_SOUND4CNT_L =   (REG_BASE + 0x78)
REG_SOUND4CNT_H =   (REG_BASE + 0x7c)

REG_WAVE_RAM    =   (REG_BASE + 0x90)   @ Sound 3 Waveform RAM
REG_WAVE_RAM0   =   (REG_BASE + 0x90)
REG_WAVE_RAM0_L =   (REG_BASE + 0x90)
REG_WAVE_RAM0_H =   (REG_BASE + 0x92)
REG_WAVE_RAM1   =   (REG_BASE + 0x94)
REG_WAVE_RAM1_L =   (REG_BASE + 0x94)
REG_WAVE_RAM1_H =   (REG_BASE + 0x96)
REG_WAVE_RAM2   =   (REG_BASE + 0x98)
REG_WAVE_RAM2_L =   (REG_BASE + 0x98)
REG_WAVE_RAM2_H =   (REG_BASE + 0x9a)
REG_WAVE_RAM3   =   (REG_BASE + 0x9c)
REG_WAVE_RAM3_L =   (REG_BASE + 0x9c)
REG_WAVE_RAM3_H =   (REG_BASE + 0x9e)

REG_FIFO        =   (REG_BASE + 0xa0)   @ Sound FIFO
REG_FIFO_A      =   (REG_BASE + 0xa0)
REG_FIFO_A_L    =   (REG_BASE + 0xa0)
REG_FIFO_A_H    =   (REG_BASE + 0xa2)
REG_FIFO_B      =   (REG_BASE + 0xa4)
REG_FIFO_B_L    =   (REG_BASE + 0xa4)
REG_FIFO_B_H    =   (REG_BASE + 0xa6)


REG_DMA0        =   (REG_BASE + 0xb0)   @ DMA 0
REG_DMA0SAD     =   (REG_BASE + 0xb0)   @ (Internal Memory -> Internal Memory)
REG_DMA0SAD_L   =   (REG_BASE + 0xb0)
REG_DMA0SAD_H   =   (REG_BASE + 0xb2)
REG_DMA0DAD     =   (REG_BASE + 0xb4)
REG_DMA0DAD_L   =   (REG_BASE + 0xb4)
REG_DMA0DAD_H   =   (REG_BASE + 0xb6)
REG_DMA0CNT     =   (REG_BASE + 0xb8)
REG_DMA0CNT_L   =   (REG_BASE + 0xb8)
REG_DMA0CNT_H   =   (REG_BASE + 0xba)
REG_DMA1        =   (REG_BASE + 0xbc)   @ DMA 1
REG_DMA1SAD     =   (REG_BASE + 0xbc)   @ (Internal Memory -> Sound FIFO/Internal Memory)
REG_DMA1SAD_L   =   (REG_BASE + 0xbc)
REG_DMA1SAD_H   =   (REG_BASE + 0xbe)
REG_DMA1DAD     =   (REG_BASE + 0xc0)
REG_DMA1DAD_L   =   (REG_BASE + 0xc0)
REG_DMA1DAD_H   =   (REG_BASE + 0xc2)
REG_DMA1CNT     =   (REG_BASE + 0xc4)
REG_DMA1CNT_L   =   (REG_BASE + 0xc4)
REG_DMA1CNT_H   =   (REG_BASE + 0xc6)
REG_DMA2        =   (REG_BASE + 0xc8)   @ DMA 2
REG_DMA2SAD     =   (REG_BASE + 0xc8)   @ (Internal Memory -> Sound FIFO/Internal Memory)
REG_DMA2SAD_L   =   (REG_BASE + 0xc8)
REG_DMA2SAD_H   =   (REG_BASE + 0xca)
REG_DMA2DAD     =   (REG_BASE + 0xcc)
REG_DMA2DAD_L   =   (REG_BASE + 0xcc)
REG_DMA2DAD_H   =   (REG_BASE + 0xce)
REG_DMA2CNT     =   (REG_BASE + 0xd0)
REG_DMA2CNT_L   =   (REG_BASE + 0xd0)
REG_DMA2CNT_H   =   (REG_BASE + 0xd2)
REG_DMA3        =   (REG_BASE + 0xd4)   @ DMA 3
REG_DMA3SAD     =   (REG_BASE + 0xd4)   @ (Game Pak/Internal Memory <--> Internal Memory)
REG_DMA3SAD_L   =   (REG_BASE + 0xd4)
REG_DMA3SAD_H   =   (REG_BASE + 0xd6)
REG_DMA3DAD     =   (REG_BASE + 0xd8)
REG_DMA3DAD_L   =   (REG_BASE + 0xd8)
REG_DMA3DAD_H   =   (REG_BASE + 0xda)
REG_DMA3CNT     =   (REG_BASE + 0xdc)
REG_DMA3CNT_L   =   (REG_BASE + 0xdc)
REG_DMA3CNT_H   =   (REG_BASE + 0xde)


REG_TM0CNT      =   (REG_BASE + 0x100)  @ Timer 0
REG_TM0CNT_L    =   (REG_BASE + 0x100)
REG_TM0CNT_H    =   (REG_BASE + 0x102)
REG_TM1CNT      =   (REG_BASE + 0x104)  @ Timer 1
REG_TM1CNT_L    =   (REG_BASE + 0x104)
REG_TM1CNT_H    =   (REG_BASE + 0x106)
REG_TM2CNT      =   (REG_BASE + 0x108)  @ Timer 2
REG_TM2CNT_L    =   (REG_BASE + 0x108)
REG_TM2CNT_H    =   (REG_BASE + 0x10a)
REG_TM3CNT      =   (REG_BASE + 0x10c)  @ Timer 3
REG_TM3CNT_L    =   (REG_BASE + 0x10c)
REG_TM3CNT_H    =   (REG_BASE + 0x10e)


REG_RCNT	=   (REG_BASE + 0x134)	@ General Input/Ouput Control

REG_SIOCNT      =   (REG_BASE + 0x128)  @ Serial Communication Control
REG_SIODATA8    =   (REG_BASE + 0x12a)  @ 8 bit Serial Communication Data
REG_SIODATA32   =   (REG_BASE + 0x120)  @ 32 bit Serial Communication Data
REG_SIOMLT_RECV =   (REG_BASE + 0x120)  @ Multi-play SIO Receive Data
REG_SIOMULTI0   =   (REG_BASE + 0x120)
REG_SIOMULTI1   =   (REG_BASE + 0x122)
REG_SIOMULTI2   =   (REG_BASE + 0x124)
REG_SIOMULTI3   =   (REG_BASE + 0x126)

REG_JOYCNT      =   (REG_BASE + 0x140)  @ JOY Bus Communication Control
REG_JOYSTAT     =   (REG_BASE + 0x158)  @ JOY Bus Status
REG_JOY_RECV    =   (REG_BASE + 0x150)  @ JOY Bus Receive Data
REG_JOY_RECV_L  =   (REG_BASE + 0x150)
REG_JOY_RECV_H  =   (REG_BASE + 0x152)
REG_JOY_TRANS   =   (REG_BASE + 0x154)  @ Joy Bus Send Data
REG_JOY_TRANS_L =   (REG_BASE + 0x154)
REG_JOY_TRANS_H =   (REG_BASE + 0x156)


REG_KEYINPUT    =   (REG_BASE + 0x130)  @ Key Input
REG_KEYCNT      =   (REG_BASE + 0x132)  @ Key Control


@--------------------------------------------------------------------
@-                      Register Offset                             -
@--------------------------------------------------------------------

OFFSET_REG_PAUSE        =   0x300       @ PAUSE

OFFSET_REG_IE           =   0x200       @ Interrupt Enable
OFFSET_REG_IF           =   0x202       @ Interrupt Request

OFFSET_REG_WAITCNT      =   0x204       @ Game Pak Wait Control

OFFSET_REG_DISPCNT      =   0x0         @ Display Control

OFFSET_REG_STAT         =   0x4         @ Status
OFFSET_REG_VCOUNT       =   0x6         @ V Counter

OFFSET_REG_MOSAIC       =   0x4c        @ Mosaic Size

OFFSET_REG_BLDCNT       =   0x50        @ Blend Mode Control
OFFSET_REG_BLDALPHA     =   0x52        @ Semi-transparent Parameter
OFFSET_REG_BLDY         =   0x54        @ Brightness Change Parameter

OFFSET_REG_WINCNT       =   0x40        @ Window Control
OFFSET_REG_WIN0H        =   0x40        @ Window 0 Horizontal Area
OFFSET_REG_WIN1H        =   0x42        @ Window 1 Horizontal Area
OFFSET_REG_WIN0V        =   0x44        @ Window 0 Vertical Area
OFFSET_REG_WIN1V        =   0x46        @ Window 1 Vertical Area
OFFSET_REG_WININ        =   0x48        @ Internal Window Control
OFFSET_REG_WIN0         =   0x48        @ Window 0 Control
OFFSET_REG_WIN1         =   0x49        @ Window 1 Control
OFFSET_REG_WINOUT       =   0x4a        @ External Window Control
OFFSET_REG_OBJWIN       =   0x4b        @ OBJ Window Control


OFFSET_REG_BGCNT        =   0x8         @ BG Control
OFFSET_REG_BG0CNT       =   0x8         @ BG 0 Control
OFFSET_REG_BG1CNT       =   0xa         @ BG 1 Control
OFFSET_REG_BG2CNT       =   0xc         @ BG 2 Control
OFFSET_REG_BG3CNT       =   0xe         @ BG 3 Control
OFFSET_REG_BGOFS        =   0x10        @ BG Offset
OFFSET_REG_BG0HOFS      =   0x10        @ BG 0 H Offset
OFFSET_REG_BG0VOFS      =   0x12        @ BG 0 V Offset
OFFSET_REG_BG1HOFS      =   0x14        @ BG 1 H Offset
OFFSET_REG_BG1VOFS      =   0x16        @ BG 1 V Offset
OFFSET_REG_BG2HOFS      =   0x18        @ BG 2 H Offset
OFFSET_REG_BG2VOFS      =   0x1a        @ BG 2 V Offset
OFFSET_REG_BG3HOFS      =   0x1c        @ BG 3 H Offset
OFFSET_REG_BG3VOFS      =   0x1e        @ BG 3 V Offset
OFFSET_REG_BG2AFFINE    =   0x20        @ BG 2 Affine Transformation Parameters
OFFSET_REG_BG2PA        =   0x20        @ BG 2 Line Direction X Coordinate Difference
OFFSET_REG_BG2PB        =   0x22        @ BG 2 Vertical Direction X Coordinate Difference
OFFSET_REG_BG2PC        =   0x24        @ BG 2 Line Direction Y Coordinate Difference
OFFSET_REG_BG2PD        =   0x26        @ BG 2 Vertical Direction Y Coordinate Difference
OFFSET_REG_BG2X         =   0x28        @ BG 2 Start X Coordinate
OFFSET_REG_BG2X_L       =   0x28
OFFSET_REG_BG2X_H       =   0x2a
OFFSET_REG_BG2Y         =   0x2c        @ BG 2 Start Y Coordinate
OFFSET_REG_BG2Y_L       =   0x2c
OFFSET_REG_BG2Y_H       =   0x2e
OFFSET_REG_BG3AFFINE    =   0x30        @ BG 3 Affine Transformation Parameters
OFFSET_REG_BG3PA        =   0x30        @ BG 3 Line Direction X Coordinate Difference
OFFSET_REG_BG3PB        =   0x32        @ BG 3 Vertical Direction X Coordinate Difference
OFFSET_REG_BG3PC        =   0x34        @ BG 3 Line Direction Y Coordinate Difference
OFFSET_REG_BG3PD        =   0x36        @ BG 3 Vertical Direction Y Coordinate Difference
OFFSET_REG_BG3X         =   0x38        @ BG 3 Start X Coordinate
OFFSET_REG_BG3X_L       =   0x38
OFFSET_REG_BG3X_H       =   0x3a
OFFSET_REG_BG3Y         =   0x3c        @ BG 3 Start Y Coordinate
OFFSET_REG_BG3Y_L       =   0x3c
OFFSET_REG_BG3Y_H       =   0x3e


OFFSET_REG_SOUNDCNT     =   0x80        @ Sound Control
OFFSET_REG_SOUNDCNT_L   =   0x80
OFFSET_REG_SOUNDCNT_H   =   0x82
OFFSET_REG_SOUNDCNT_X   =   0x84

OFFSET_REG_SOUNDBIAS    =   0x88        @ Sound BIAS

OFFSET_REG_SOUND1CNT    =   0x60        @ Sound 1 Control
OFFSET_REG_SOUND1CNT_L  =   0x60
OFFSET_REG_SOUND1CNT_H  =   0x62
OFFSET_REG_SOUND1CNT_X  =   0x64
OFFSET_REG_SOUND2CNT    =   0x68        @ Sound 2 Control
OFFSET_REG_SOUND2CNT_L  =   0x68
OFFSET_REG_SOUND2CNT_H  =   0x6c
OFFSET_REG_SOUND3CNT    =   0x70        @ Sound 3 Control
OFFSET_REG_SOUND3CNT_L  =   0x70
OFFSET_REG_SOUND3CNT_H  =   0x72
OFFSET_REG_SOUND3CNT_X  =   0x74
OFFSET_REG_SOUND4CNT    =   0x78        @ Sound 4 Control
OFFSET_REG_SOUND4CNT_L  =   0x78
OFFSET_REG_SOUND4CNT_H  =   0x7c

OFFSET_REG_WAVE_RAM     =   0x90        @ Sound 3 Waveform RAM
OFFSET_REG_WAVE_RAM0    =   0x90
OFFSET_REG_WAVE_RAM0_L  =   0x90
OFFSET_REG_WAVE_RAM0_H  =   0x92
OFFSET_REG_WAVE_RAM1    =   0x94
OFFSET_REG_WAVE_RAM1_L  =   0x94
OFFSET_REG_WAVE_RAM1_H  =   0x96
OFFSET_REG_WAVE_RAM2    =   0x98
OFFSET_REG_WAVE_RAM2_L  =   0x98
OFFSET_REG_WAVE_RAM2_H  =   0x9a
OFFSET_REG_WAVE_RAM3    =   0x9c
OFFSET_REG_WAVE_RAM3_L  =   0x9c
OFFSET_REG_WAVE_RAM3_H  =   0x9e

OFFSET_REG_FIFO         =   0xa0        @ Sound FIFO
OFFSET_REG_FIFO_A       =   0xa0
OFFSET_REG_FIFO_A_L     =   0xa0
OFFSET_REG_FIFO_A_H     =   0xa2
OFFSET_REG_FIFO_B       =   0xa4
OFFSET_REG_FIFO_B_L     =   0xa4
OFFSET_REG_FIFO_B_H     =   0xa6


OFFSET_REG_DMA0         =   0xb0        @ DMA 0
OFFSET_REG_DMA0SAD      =   0xb0        @ (Internal Memory -> Internal Memory)
OFFSET_REG_DMA0SAD_L    =   0xb0
OFFSET_REG_DMA0SAD_H    =   0xb2
OFFSET_REG_DMA0DAD      =   0xb4
OFFSET_REG_DMA0DAD_L    =   0xb4
OFFSET_REG_DMA0DAD_H    =   0xb6
OFFSET_REG_DMA0CNT      =   0xb8
OFFSET_REG_DMA0CNT_L    =   0xb8
OFFSET_REG_DMA0CNT_H    =   0xba
OFFSET_REG_DMA1         =   0xbc        @ DMA 1
OFFSET_REG_DMA1SAD      =   0xbc        @ (Internal Memory -> Sound FIFO/Internal Memory)
OFFSET_REG_DMA1SAD_L    =   0xbc
OFFSET_REG_DMA1SAD_H    =   0xbe
OFFSET_REG_DMA1DAD      =   0xc0
OFFSET_REG_DMA1DAD_L    =   0xc0
OFFSET_REG_DMA1DAD_H    =   0xc2
OFFSET_REG_DMA1CNT      =   0xc4
OFFSET_REG_DMA1CNT_L    =   0xc4
OFFSET_REG_DMA1CNT_H    =   0xc6
OFFSET_REG_DMA2         =   0xc8        @ DMA 2
OFFSET_REG_DMA2SAD      =   0xc8        @ (Internal Memory -> Sound FIFO/Internal Memory)
OFFSET_REG_DMA2SAD_L    =   0xc8
OFFSET_REG_DMA2SAD_H    =   0xca
OFFSET_REG_DMA2DAD      =   0xcc
OFFSET_REG_DMA2DAD_L    =   0xcc
OFFSET_REG_DMA2DAD_H    =   0xce
OFFSET_REG_DMA2CNT      =   0xd0
OFFSET_REG_DMA2CNT_L    =   0xd0
OFFSET_REG_DMA2CNT_H    =   0xd2
OFFSET_REG_DMA3         =   0xd4        @ DMA 3
OFFSET_REG_DMA3SAD      =   0xd4        @ (Game Pak/Internal Memory <--> Internal Memory)
OFFSET_REG_DMA3SAD_L    =   0xd4
OFFSET_REG_DMA3SAD_H    =   0xd6
OFFSET_REG_DMA3DAD      =   0xd8
OFFSET_REG_DMA3DAD_L    =   0xd8
OFFSET_REG_DMA3DAD_H    =   0xda
OFFSET_REG_DMA3CNT      =   0xdc
OFFSET_REG_DMA3CNT_L    =   0xdc
OFFSET_REG_DMA3CNT_H    =   0xde


OFFSET_REG_TM0CNT       =   0x100       @ Timer 0
OFFSET_REG_TM0CNT_L     =   0x100
OFFSET_REG_TM0CNT_H     =   0x102
OFFSET_REG_TM1CNT       =   0x104       @ Timer 1
OFFSET_REG_TM1CNT_L     =   0x104
OFFSET_REG_TM1CNT_H     =   0x106
OFFSET_REG_TM2CNT       =   0x108       @ Timer 2
OFFSET_REG_TM2CNT_L     =   0x108
OFFSET_REG_TM2CNT_H     =   0x10a
OFFSET_REG_TM3CNT       =   0x10c       @ Timer 3
OFFSET_REG_TM3CNT_L     =   0x10c
OFFSET_REG_TM3CNT_H     =   0x10e


OFFSET_REG_RCNT         =   0x134       @ General Input/Output Control

OFFSET_REG_SIOCNT       =   0x128       @ Serial Communication Control
OFFSET_REG_SIODATA8     =   0x12a       @ 8 bit Serial Communication Data
OFFSET_REG_SIODATA32    =   0x120       @ 32 bit Serial Communication Data
OFFSET_REG_SIOMLT_RECV  =   0x120       @ Multi-play SIO Receive Data
OFFSET_REG_SIOMULTI0    =   0x120
OFFSET_REG_SIOMULTI1    =   0x122
OFFSET_REG_SIOMULTI2    =   0x124
OFFSET_REG_SIOMULTI3    =   0x126

OFFSET_REG_JOYCNT       =   0x140       @ JOY Bus Communication Control
OFFSET_REG_JOYSTAT      =   0x158       @ JOY Bus Status
OFFSET_REG_JOY_RECV     =   0x150       @ JOY Bus Receive Data
OFFSET_REG_JOY_RECV_L   =   0x150
OFFSET_REG_JOY_RECV_H   =   0x152
OFFSET_REG_JOY_TRANS    =   0x154       @ JOY Bus Send Data
OFFSET_REG_JOY_TRANS_L  =   0x154
OFFSET_REG_JOY_TRANS_H  =   0x156


OFFSET_REG_KEYINPUT     =   0x130       @ Key Input
OFFSET_REG_KEYCNT       =   0x132       @ Key Control

