//#include "registers.h"
// small selection of registers follows...


// Clock & power management
#define rLOCKTIME		(*(volatile unsigned long *)0x14800000)		// R/W PLL Lock Time Counter
#define rMPLLCON		(*(volatile unsigned long *)0x14800004)		// R/W MPLL Control
#define rUPLLCON		(*(volatile unsigned long *)0x14800008)		// R/W UPLL Control
#define rCLKCON			(*(volatile unsigned long *)0x1480000C)		// R/W Clock Generator Control
#define rCLKSLOW		(*(volatile unsigned long *)0x14800010)		// R/W Slow Clock Control
#define rCLKDIVN		(*(volatile unsigned long *)0x14800014)		// R/W Clock divider Control

// UART 0 
#define rULCON0			(*(volatile unsigned long *)0x15000000)		// R/W UART 0 Line Control
#define rUCON0			(*(volatile unsigned long *)0x15000004)		// R/W UART 0 Control
#define rUFCON0			(*(volatile unsigned long *)0x15000008)		// R/W UART 0 FIFO Control
#define rUMCON0			(*(volatile unsigned long *)0x1500000c)		// R/W UART 0 Modem Control
#define rUTRSTAT0		(*(volatile unsigned long *)0x15000010)		// R UART 0 Tx/Rx Status
#define rUERSTAT0		(*(volatile unsigned long *)0x15000014)		// R UART 0 Rx Error Status
#define rUFSTAT0		(*(volatile unsigned long *)0x15000018)		// R UART 0 FIFO Status
#define rUMSTAT0		(*(volatile unsigned long *)0x1500001c)		// R UART 0 Modem Status
#define rUTXH0			(*(volatile unsigned char *)0x15000020)		// W UART 0 Transmission Hold
#define rURXH0			(*(volatile unsigned char *)0x15000024)		// R UART 0 Receive Buffer
#define rUBRDIV0		(*(volatile unsigned long *)0x15000028)		// R/W UART 0 Baud Rate Divisor




unsigned char UARTRecvByte()
{
	while ((rUFSTAT0 & 0x7) == 0);		// wait until any byte in RX FIFO
	return rURXH0;
}

unsigned int UARTRecvLong()
{
	unsigned long d0 = UARTRecvByte();
	unsigned long d1 = UARTRecvByte();
	unsigned long d2 = UARTRecvByte();
	unsigned long d3 = UARTRecvByte();
	return d0 | d1<<8 | d2<<16 | d3<<24;
}

void UARTSendByte(unsigned char c)
{
	while (rUFSTAT0 & (1<<9));	// loop when TX FIFO is full
	rUTXH0 = c;
}

void UARTSendString(char *s)
{
	while (*s) UARTSendByte(*s++);
}


void UARTSendHexNibble(unsigned char x)
{
	const char hexTable[] = "0123456789ABCDEF";
	UARTSendByte(hexTable[x]);
}

void UARTSendHexByte(unsigned char x)
{
	UARTSendHexNibble(x >> 4);
	UARTSendHexNibble(x & 0x0F);
}

void UARTSendHexShort(unsigned short x)
{
	UARTSendHexByte(x >> 8);
	UARTSendHexByte((unsigned char)x);
}

void UARTSendHexLong(unsigned long x)
{
	UARTSendHexShort(x >> 16);
	UARTSendHexShort((unsigned short)x);
}


void Init()
{
	//ClockSetSpeed(0x3a, 0x1, 0x1, 2);	// 3ah, 1h,1h |132000000|264M|  4.0M| 0.2| 0.3| 0.4|1|	ratio 2
	const unsigned char mdiv = 0x3a;
	const unsigned char pdiv = 0x1;
	const unsigned char sdiv = 0x1;
	const unsigned char ratio = 2;
	const unsigned int f = ((mdiv + 8) * 12000000) / ((pdiv + 2) * (1<<sdiv));
	const unsigned int h = (ratio & 2) ? f / 2 : f;
	const unsigned int p = (ratio & 1) ? h / 2 : h;
	rCLKDIVN = ratio;
	rMPLLCON = mdiv<<12|pdiv<<4|sdiv;

	// UARTInit(115200);
	const unsigned int baudrate = 115200;
	rCLKCON		|= 0x100;
	rULCON0		= 0x03;	// 8 bits, no-parity
	rUCON0		= 0x05;	// polled mode
	rUFCON0		= 0xE7;	// enable FIFO
	rUMCON0		= 0x00;	// no handshaking
	rUBRDIV0	= (unsigned int)(p / (baudrate * 16)) - 1;


	UARTSendString("Hello world!\n");
}
