;
; Standard joystick driver for the NES
;
; Groepaz/Hitmen, 2003
;

		.include 	"zeropage.inc"

      	.include 	"joy-kernel.inc"
        .include    "joy-error.inc"
        .include    "nes.inc"

        .macpack        generic

; ------------------------------------------------------------------------
; Header. Includes jump table

.segment        "CODE"

; Driver signature

;;        .byte   $6A, $6F, $79		; "joy"
;;        .byte   $00                     ; Driver API version number

; Button state masks (8 values)

;extern const unsigned char joy_masks[8];

		.export _joy_masks

_joy_masks:
        .byte   $01                     ; JOY_UP
        .byte   $02                     ; JOY_DOWN
        .byte   $04                     ; JOY_LEFT
        .byte   $08                     ; JOY_RIGHT
        .byte   $10                     ; JOY_FIRE
        .byte   $20                     ; B                   
        .byte   $40                     ; Select           
        .byte   $80                     ; Start                  

; Jump table.

;;        .word   INSTALL
;;        .word   DEINSTALL
;;        .word   COUNT
;;        .word   READ

; ------------------------------------------------------------------------
; Constants

JOY_COUNT       = 2             ; Number of joysticks we support


; ------------------------------------------------------------------------
; Data.


.code


;extern const char joy_stddrv[];

		.export _joy_stddrv
_joy_stddrv:
		.byte 0


		.export _joy_load_driver
		.export _joy_unload

;unsigned char __fastcall__ joy_unload (void);
;unsigned char __fastcall__ joy_load_driver (const char* driver);
_joy_load_driver:
_joy_unload:

; ------------------------------------------------------------------------
; INSTALL routine. Is called after the driver is loaded into memory. If
; possible, check if the hardware is present and determine the amount of
; memory available.
; Must return an JOY_ERR_xx code in a/x.
;

INSTALL:
        lda     #<JOY_ERR_OK
        ldx     #>JOY_ERR_OK

;	rts                     ; Run into DEINSTALL instead

; ------------------------------------------------------------------------
; DEINSTALL routine. Is called before the driver is removed from memory.
; Can do cleanup or whatever. Must not return anything.
;

DEINSTALL:
        rts


; ------------------------------------------------------------------------
; COUNT: Return the total number of available joysticks in a/x.
;
;unsigned char __fastcall__ joy_count (void);

		.export _joy_count

_joy_count:
COUNT:
        lda     #<JOY_COUNT
        ldx     #>JOY_COUNT
        rts

; ------------------------------------------------------------------------
; READ: Read a particular joystick passed in A.
;
;unsigned char __fastcall__ joy_read (unsigned char joystick);

		.export _joy_read

_joy_read:
READ:
		; reset controllers
        ldy #$01
        sty APU_PAD1
        ldy #$00
        sty APU_PAD1

		tax		; Joystick number into X
		bne    	joy2

; Read joystick 1

joy1:

		ldy #0
		ldx #0
@l:
		lda APU_PAD1
		and #$01
		beq @s
        tya
		ora joybits,x
		tay
@s:
		inx
		cpx #8
		bne @l

		ldx #0
		tya
     	rts

; Read joystick 2

joy2:

		ldy #0
		ldx #0
@l:
		lda APU_PAD2
		and #$01
		beq @s
        tya
		ora joybits,x
		tay
@s:
		inx
		cpx #8
		bne @l

		ldx #0
		tya
		rts

 ;    bit  0 : button b
 ;    bit  1 : button y
 ;    bit  2 : select
 ;    bit  3 : start
 ;    bit  4 : up
 ;    bit  5 : down
 ;    bit  6 : left
 ;    bit  7 : right
 
 ;    bit  8 : button a
 ;    bit  9 : button x
 ;    bit 10 : top-left l
 ;    bit 11 : top-right r
 ;    bit 12+: always 1

.rodata
joybits:
        .byte   $10                     ; JOY_FIRE
        .byte   $20                     ; (B)
        .byte   $40                     ; (Select)
        .byte   $80                     ; (start)
        .byte   $01                     ; JOY_UP
        .byte   $02                     ; JOY_DOWN
        .byte   $04                     ; JOY_LEFT
        .byte   $08                     ; JOY_RIGHT

